/***

KameleonDUI - DUI-local.m
by Leechbite

script to control the DUI interfaces of local windows (e.g. EQ, CD Cover etc.)

***/

#include <lib/std.mi>
#include "attribs.m"
#include "utilities.m"
#include "dui-utilities.m"

function requestOpen();
function requestClose();

function duiClose();
function windowOpen();

function setToDUIMode();
function resetFromDUIMode();

function toggleComponent(string compID);

global container localContainer;
global layout localLayout;
global guiobject concealer;

global container duiMainContainer;
global layout duiMainLayout;
global group duiMainGroup;
global guiObject duiMainWin, duiMainObjCont;

Global String duix, duiy, duiw, duih;
Global string duimin_w, duimin_h, duimax_w, duimax_h;
Global String windowName, groupid, duiWindow, defDuiWindow, contName, layName;
global int localOpen = FALSE, openLastResizeHide = FALSE, localLayShow = FALSE;

global ConfigAttribute currwindow_attach_attrib, currwindow_duiwindow_attrib; //, currwindow_tempdetach_attrib;

global button attachButton,detachButton,windowButton,closeButton;

global timer onStartTimer, delayOpen, delayClose, delayToDUI;

System.onScriptLoaded() {
	initAttribs();
	
	string param = getParam();
	windowName = getToken(param,",",0);
	groupid = getToken(param,",",1);
	defDuiWindow = strlower(getToken(param, ",", 2));
	duiWindow = defDuiWindow;
	
	setPrivateString(getSkinName(),"Def DUI:"+windowName,defDuiWindow);
	
	duimin_w = getToken(param,",",3);
	duimin_h = getToken(param,",",4);
	duimax_w = getToken(param,",",5);
	duimax_h = getToken(param,",",6);
	contName = getToken(param,",",7);
	layName = getToken(param,",",8);
	
	delayOpen = new Timer;
	delayOpen.setDelay(50);
	
	delayClose = new Timer;
	delayClose.setDelay(1);
	
	delayToDUI = new Timer;
	delayToDUI.setDelay(1);

	onStartTimer = new Timer;
	onStartTimer.setDelay(1);
	onStartTimer.start();
	
}
// **** DUI scripts
system.onScriptUnloading() {
	delete onStartTimer;
	delete delayOpen;
	delete delayClose;
	delete delayToDUI;
	
	if (dui_temp_detach_attrib.getData()=="0") {
		if (duiMainObjCont) 
			setPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,(duiMainObjCont.getXMLParam("groupid") == groupid)); // show on skin load
		else
			setPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,0);
	}
	
	if (duiMainObjCont) {
		if (duiMainObjCont.getXMLParam("groupid") == groupid)
			sendDUICommand(windowName, CLOSE_DUI_WINDOW, duiWindow);
	}

}

onStartTimer.onTimer() {
	stop();
	
	localContainer = getContainer(contName);
	localLayout = localContainer.getLayout(layName);
	concealer = localLayout.findObject("frame.concealer");
	
	sendDUICommand(windowName, ADD_TO_DUI_LIST, "");
	currwindow_attach_attrib = custom_page_windows.newAttribute("Attach "+windowName+" window", "1");
	currwindow_duiwindow_attrib = custom_page_nonexposed.newAttribute("DUI window "+windowName, defDuiWindow);
	duiWindow = currwindow_duiwindow_attrib.getData();
	
	if (currwindow_attach_attrib.getData()=="1") concealer.show();
	
	if (localLayout.isVisible()) localLayout.onSetVisible(1);
	
	if (dui_temp_detach_attrib.getData()=="0") {
		if (getPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,0)==1) currwindow_attach_attrib.onDataChanged();
		setPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,0);
	}
}

localLayout.onSetVisible(int on) {
	if (localLayShow) return;
	if (onStartTimer.isRunning()) return;
	
	if (on) {
		if (currwindow_attach_attrib) {
			if ((currwindow_attach_attrib.getData()!="1") || (dui_temp_detach_attrib.getData()=="1")) {
				concealer.hide();
			} else {
				requestOpen();
			}
		}
	} else {
		concealer.show();
		if (currwindow_attach_attrib) {
			if ((currwindow_attach_attrib.getData()=="1") || (dui_temp_detach_attrib.getData()!="1")) {
				requestClose();
			}
		}
	}
}

requestOpen() {
	openLastResizeHide = FALSE;
	if (dui_temp_detach_attrib.getData()=="1") {
		setPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,1);
		concealer.show();
		localLayout.hide();
		return;
	}

	duiWindow = currwindow_duiwindow_attrib.getData();
	localLayShow = TRUE;
	concealer.show();
	localLayout.show();
	localLayShow = FALSE;

	int iduimin_w = stringToInteger(duimin_w);
	int iduimin_h = stringToInteger(duimin_h);
	int defw = getPrivateInt(getSkinName(),"duidefW "+windowName,iduimin_w);
	int defh = getPrivateInt(getSkinName(),"duidefH "+windowName,iduimin_h);
	if (defw < iduimin_w) defw = iduimin_w; // just check for min, this is just for contengency
	if (defh < iduimin_h) defh = iduimin_h;
	
	sendDUICommand(windowName, OPEN_DUI_REQUEST, duiWindow+","+duimin_w+","+duimin_h+","+duimax_w+","+duimax_h+","+
							integerToString(defw)+","+integerToString(defh));
}

delayOpen.onTimer() {
	stop();
	
	requestOpen();
}

requestClose() {
	if ((currwindow_attach_attrib.getData()=="1") || (dui_temp_detach_attrib.getData()=="1")) {
		concealer.show();
		localLayout.hide();
	}
	
	sendDUICommand(windowName, CLOSE_DUI_WINDOW, duiWindow);
	
	duiMainContainer = NULL;
	duiMainLayout = NULL;
	duiMainGroup = NULL;
	duiMainGroup = NULL;
	duiMainWin = NULL;

	delayClose.start();
}

duiClose() {
	if (duiMainObjCont) {
		if (duiMainObjCont.getXMLParam("groupid") == groupid) duiMainObjCont.setXMLParam("groupid","dui.empty");
	}

	duiMainObjCont = NULL;
}

delayClose.onTimer() {
	stop();
	
	duiClose();
}

currwindow_attach_attrib.onDataChanged() {
	if (getData()=="1") {
		if (localLayout.isVisible() || (getPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,0)==1)) {
			requestOpen();
		}
	} else {
		if (duiMainObjCont) {
			if (duiMainObjCont.getXMLParam("groupid") == groupid) {
				string nextcomp = getPrivateString(getSkinName(),strupper(windowName)+" Next Window","");
				if ((windows_duitab_attrib.getData()=="1") && (nextcomp!="") && (dui_temp_detach_attrib.getData()!="1")) {
					toggleComponent(nextcomp);
				} else
					requestClose();
					
				windowOpen();
			}
		}
	}
}

windowOpen() {
	localLayout.show();	
	concealer.hide();
	localLayout.setFocus();
}

attachButton.onLeftClick() {
	if (currwindow_attach_attrib) currwindow_attach_attrib.setData("1");
}

detachButton.onLeftClick() {
	if (currwindow_attach_attrib) currwindow_attach_attrib.setData("0");
}

closeButton.onLeftClick() {
	requestClose();
}

windowButton.onLeftClick() {
	if (!currwindow_duiwindow_attrib) return;
	
	popupMenu DUIwindowMenu;
	
	string currDUIWin = currwindow_duiwindow_attrib.getData();
	
	DUIwindowMenu = new popupMenu;
	DUIwindowMenu.addCommand("- Select DUI Window -", -1, 0, 1);
	DUIwindowMenu.addSeparator();
	DUIwindowMenu.addCommand(" Main", 1, currDUIWin==DUI_WIN_MAIN, 0);
	DUIwindowMenu.addCommand(" Bottom Main", 2, currDUIWin==DUI_WIN_MAIN2, 0);
	DUIwindowMenu.addCommand(" Side", 3, currDUIWin==DUI_WIN_SIDE, 0);
	DUIwindowMenu.addCommand(" Top Side", 4, currDUIWin==DUI_WIN_SIDE2, 0);
	DUIwindowMenu.addSeparator();
	DUIwindowMenu.addCommand(" Reset to default", 5, 0, 0);
	
	string lastTip = getXMLParam("tooltip");
	setXMLParam("tooltip", "");

  	int res = DUIwindowMenu.popAtMouse();
  	delete DUIwindowMenu;
  	
  	if (res == 1) currwindow_duiwindow_attrib.setData(DUI_WIN_MAIN);
  	if (res == 2) currwindow_duiwindow_attrib.setData(DUI_WIN_MAIN2);
  	if (res == 3) currwindow_duiwindow_attrib.setData(DUI_WIN_SIDE);
  	if (res == 4) currwindow_duiwindow_attrib.setData(DUI_WIN_SIDE2);
  	if (res == 5) currwindow_duiwindow_attrib.setData(defDuiWindow);
  	
  	setXMLParam("tooltip", lastTip);
}

currwindow_duiwindow_attrib.onDataChanged() {
	string lastDUIWindow = duiWindow;
	duiWindow = getData();
	
	if (lastDUIWindow == duiWindow) return;
	if (!currwindow_duiwindow_attrib) return;
	if (!duiMainObjCont) return;
	if (duiMainObjCont.getXMLParam("groupid") != groupid) return;

	string nextcomp = getPrivateString(getSkinName(),strupper(windowName)+" Next Window","");
	
	if ((windows_duitab_attrib.getData()=="1") && (nextcomp!="")) {
		toggleComponent(nextcomp);
	} else
		sendDUICommand(windowName, CLOSE_DUI_WINDOW, lastDUIWindow);
		
	delayOpen.start();
}

dui_temp_detach_attrib.onDataChanged() {
	if (!currwindow_attach_attrib) return;

	if (currwindow_attach_attrib.getData()=="0") {  // if not attached, dont bother to detach
		setPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,0);
		return;
	}

	if (getData()=="0") {
		currwindow_attach_attrib.onDataChanged(); // reattach
	} else {
		setPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,0);
		if (duiMainObjCont) {
			if (duiMainObjCont.getXMLParam("groupid") == groupid) { // check if this is on dui window
				requestClose(); // close dui window
				
				if (windows_detachonshade_attrib.getData()=="1") // open detached window	
					windowOpen();
				else {
					setPrivateInt(getSkinName(),"OpenOnLastDUI "+windowName,1);
				}

			}
		}
	}
}

dui_onresizehide_attrib.onDataChanged() {
	if (getData()=="1") {
		openLastResizeHide = FALSE;
		if (duiMainObjCont) {
			if (duiMainObjCont.getXMLParam("groupid") == groupid) {
				delayClose.start();
				localLayShow = TRUE;
				localLayout.hide();
				localLayShow = FALSE;
				openLastResizeHide = TRUE;
			}
		}
	} else {
		string lowDUIWin = strlower(currwindow_duiwindow_attrib.getData());
		if ((strlower(getPrivateString(getSkinName(),"onDUIClosing",""))!=lowDUIWin) && openLastResizeHide) {
			delayOpen.start();
		}


		openLastResizeHide = FALSE;
	}
}

onReceiveDUICommand(string winName, string command, string param) {
	
	string lowWinName = strlower(winName);
	string lowWinName2 = strlower(windowName);
	if (lowWinName == lowWinName2) return;
	
	string param1 = strlower(gettoken(param,",",0));
	
	if (winName == MAINID) {
	
		if (command == OPEN_DUI_GRANTED) {
		  if (currwindow_attach_attrib) { 
			  if ((currwindow_attach_attrib.getData()=="0") && (dui_temp_detach_attrib.getData()=="0")) return; }
			  
		  if (param1 == "all") param1 = strlower(windowName);
		  if (param1 == strlower(windowName)) {

		    string duiContainerID = strlower(gettoken(param,",",1));
		    string duiLayoutID = strlower(gettoken(param,",",2));
		    string duiGroupID = strlower(gettoken(param,",",3));
		    string duiWinID = strlower(gettoken(param,",",4));
		    
		    duiMainContainer = getContainer(duiContainerID);
		    if (duiMainContainer) duiMainLayout = duiMainContainer.getLayout(duiLayoutID);
		    if (duiMainLayout) duiMainGroup = duiMainLayout.findObject(duiGroupID);
		    if (duiMainGroup) duiMainWin = duiMainGroup.findObject(duiWinID);
		    if (duiMainGroup) duiMainObjCont = duiMainGroup.findObject("obj.container");
		    if (duiMainObjCont) duiMainObjCont.show();
		    if (duiMainWin) duiMainWin.hide();

		    delayToDUI.start();
		    
		  }
		} else if (command == DETACH_WINDOW_GRANTED) { // probably wont be used.
		  if (param1 == "all") param1 = strlower(windowName);
		  if (param1 == strlower(windowName)) {
		    duiClose();
		    windowOpen();
		  }
		} else if (command == CLOSE_DUI_WINDOW) {
			if (param1 == "all") param1 = strlower(windowName);

			if (param1 == strlower(windowName)) {
				if (duiMainObjCont) {
					if ((currwindow_attach_attrib.getData()=="1") || (dui_temp_detach_attrib.getData()=="1")) {
						concealer.show();
						localLayout.hide();
					}
					delayClose.start();
					
				}
			}
		}
	}
}


duiMainWin.onResize(int wx, int wy, int ww, int wh) {
	if (dui_on_transition_attrib.getData() == "1") return;
	if (!duiMainGroup) return;
	if (!duiMainObjCont) return;
	if (duiMainObjCont.getXMLParam("groupid") != groupid) return;
	
	setPrivateInt(getSkinName(),"duidefW "+windowName,duiMainWin.getWidth());
	setPrivateInt(getSkinName(),"duidefH "+windowName,duiMainWin.getHeight());
}

delayToDUI.onTimer() {
	stop();
	
	setToDUIMode();
}

setToDUIMode() {
	if (!duiMainObjCont) return;
	
	duiMainObjCont.setXMLParam("groupid",groupid);
	
	detachButton = duiMainObjCont.findObject("dui.detach");
	windowButton = duiMainObjCont.findObject("dui.window");
	closeButton = duiMainObjCont.findObject("dui.close2");

}

toggleComponent(string compID) {
	layout mainLayout = getContainer("main").getLayout("normal");

	if (compID=="PL") {
		triggerAction(mainLayout,"TOGGLE","guid:pl");
	} else if (compID=="ML") {
		triggerAction(mainLayout,"TOGGLE","guid:ml");
	} else if (compID=="VID") {
		triggerAction(mainLayout,"TOGGLE","guid:{F0816D7B-FFFC-4343-80F2-E8199AA15CC3}");
	} else if (compID=="VIS") {
		triggerAction(mainLayout,"TOGGLE","guid:avs");
	} else if (compID=="EQ") {
		triggerAction(mainLayout,"TOGGLE","eq");
	} else if (compID=="IE") {
		triggerAction(mainLayout,"TOGGLE","ibrowser");
	} else if (compID=="CD") {
		triggerAction(mainLayout,"TOGGLE","cdcover");
	} else if (compID=="CP") {
		triggerAction(mainLayout,"TOGGLE","Config");
	}
}
