/***

Dynamic User Interface v2.0 - DUI-Main.m
by Leechbite

main script for the DUI interface.

***/

#include <lib/std.mi>
#include "attribs.m"
#include "utilities.m"
#include "dui-utilities.m"

function setNewGroup(String groupid);
function openDUIWindow(string duiWinID, string windowName, int min_w, int min_h, int max_w, int max_h, int def_w, int def_h);
function closeDUIWindow(string duiWinID);
function closeDUIWindowContent(string duiWinID, string windowName);
function resizeDUIWindow(string duiWinID, int neww, int newh);
function ConfigAttribute getDUIwinAttrib(string duiWinID);
function string getMainDUIWin(string duiWinID);
function initDUIAttrib(string windowName);

function refreshDUILayout(); // refreshes the DUI dimensions
function int getMainWidth(); // computes current width of main DUI window based on sideWidth
function int getMainMinWidth(); // computes for the minimum width of main & main2 windows
function int getMainMinHeight(); // computes for the minimum height of main & main2 windows
function int getSideMinWidth(); // computes for the minimum width of side & side2 windows
function int getSideMinHeight(); // computes for the minimum height of side & side2 windows
function int getMinWidth(); // computes the proper minimum layout width for the current displayed windows
function int getMinHeight(); // ditto.. except its for height.

Global group frameGroup;
Global layout duiLayout;
Global container duiContainer;

Global group duiWindowMain,duiWindowSide;
Global guiObject duiWindowGroupMain,duiWindowGroupMain2,duiWindowGroupSide,duiWindowGroupSide2;
Global guiObject duiWindowContMain,duiWindowContMain2,duiWindowContSide,duiWindowContSide2;
Global string duiLay_min_w, duiLay_min_h, duiLay_max_w, duiLay_max_h;
Global int main_min_w, main_min_h, main_max_w, main_max_h;
Global int main2_min_w, main2_min_h, main2_max_w, main2_max_h;
Global int side_min_w, side_min_h, side_max_w, side_max_h;
Global int side2_min_w, side2_min_h, side2_max_w, side2_max_h;
Global int sideWidth, side2Height, main2Height;
Global int duiHoleToGroupDiffWidth = 0, duiHoleToGroupDiffHeight = 0;
Global boolean localDUIOpenClose = FALSE, resizerDown = FALSE;

Global timer sendDUIDelay;
Global string sendDUIWindowName, sendDUICommandParam, sendDUIParam;

Global string windowList;

global group resizerHorizGroup;
global guiObject resizerHorizHandle;

global group resizerMainGroup;
global guiObject resizerMainHandle;

global group resizerSideGroup;
global guiObject resizerSideHandle;

global boolean localFocus = FALSE;
global boolean lastFocus = FALSE;
global int localResize, minLayoutH, minLayoutW, resizeNewH, resizeNewW;

Global ConfigAttribute duiAttrib;

Global timer onLoadTimer, resizeTimer;

System.onScriptLoaded() {
  initAttribs();
	
  frameGroup = getScriptGroup();
  duiLayout = frameGroup.getParentLayout();
  duiContainer = duiLayout.getContainer();
  
  duiWindowMain = frameGroup.findObject("dui.main");
  duiWindowSide = frameGroup.findObject("dui.side");
  
  duiWindowMain.hide();
  duiWindowSide.hide();

  duiWindowGroupMain = duiWindowMain.findObject("dui.main.content");
  duiWindowGroupSide = duiWindowSide.findObject("dui.side.content");
  duiWindowGroupMain2 = duiWindowMain.findObject("dui.main2.content");
  duiWindowGroupSide2 = duiWindowSide.findObject("dui.side2.content");
  
  duiWindowContMain = duiWindowGroupMain.findObject("obj.container");
  duiWindowContMain2 = duiWindowGroupMain2.findObject("obj.container");
  duiWindowContSide = duiWindowGroupSide.findObject("obj.container");
  duiWindowContSide2 = duiWindowGroupSide2.findObject("obj.container");

  resizerHorizGroup = frameGroup.findObject("dui.resizer.horizontal");
  resizerHorizHandle = resizerHorizGroup.findObject("resizer.handle");
  resizerMainGroup = duiWindowMain.findObject("dui.resizer.vertical");
  resizerMainHandle = resizerMainGroup.findObject("resizer.handle");
  resizerSideGroup = duiWindowSide.findObject("dui.resizer.vertical");
  resizerSideHandle = resizerSideGroup.findObject("resizer.handle");

  sideWidth = getPrivateInt(getSkinName(),"Side DUI Window Width",150);
  side2Height = getPrivateInt(getSkinName(),"Side2 DUI Window Height",150);
  main2Height = getPrivateInt(getSkinName(),"Main2 DUI Window Height",150);
  
  duiLay_min_w = duiLayout.getXMLParam("minimum_w");
  duiLay_min_h = duiLayout.getXMLParam("minimum_h");
  duiLay_max_w = duiLayout.getXMLParam("maximum_w");
  duiLay_max_h = duiLayout.getXMLParam("maximum_h");
  
  main_min_w = 50; main_min_h = 50; main_max_w = 2000; main_max_h = 2000;
  main2_min_w = 50; main2_min_h = 50; main2_max_w = 2000; main2_max_h = 2000;
  side_min_w = 50; side_min_h = 50; side_max_w = 2000; side_max_h = 2000;
  side2_min_w = 50; side2_min_h = 50; side2_max_w = 2000; side2_max_h = 2000;
  minLayoutH = stringToInteger(duiLay_min_h);
    
  sendDUIDelay = new Timer; // timer used to send a DUI command but finishes up the current script block....
  sendDUIDelay.setDelay(1);

  if (getPrivateString(getSkinName(), "DUI Window List", "") == "") {
	  setPrivateString(getSkinName(), "DUI Window List", "Playlist Editor,Media Library,Video,Visualizations,Equalizer,Internet Browser,CD Cover Art,Config Panel");
  }
  windowList = getPrivateString(getSkinName(), "DUI Window List", "");
  int c = 0;
  string winName = "";
  do {
    winName = getToken(windowList, ",", c);
    if (winName!="") {
      initDUIAttrib(winName);
    }
    c++;
  } while (winName!="");
  
  if (getPrivateInt(getSkinName(),MAINID+" X",0) > getViewPortWidth()) setPrivateInt(getSkinName(),MAINID+" X",100);
  if (getPrivateInt(getSkinName(),MAINID+" Y",0) > getViewPortHeight()) setPrivateInt(getSkinName(),MAINID+" Y",100);
  duiLayout.resize(getPrivateInt(getSkinName(),MAINID+" X",duiLayout.getLeft()),
		getPrivateInt(getSkinName(),MAINID+" Y",duiLayout.getTop()),
		getPrivateInt(getSkinName(),MAINID+" W",duiLayout.getWidth()), stringToInteger(duiLay_min_h));
		//getPrivateInt(getSkinName(),MAINID+" H",duiLayout.getHeight()));
		
  onLoadTimer = new Timer;
  onLoadTimer.setDelay(100);
  onLoadTimer.start();
  
  resizeTimer = new Timer;
  resizeTimer.setDelay(1);
  
  duiLayout.onSetVisible(duiLayout.isVisible());
}

// delayed onload scripts, used to let winamp have time to establish the containers and layouts
onLoadTimer.onTimer() {
  stop();
  refreshDUILayout();
}

System.onScriptUnloading() {
	setPrivateInt(getSkinName(),"Side DUI Window Width",sideWidth);
	setPrivateInt(getSkinName(),"Side2 DUI Window Height",side2Height);
	setPrivateInt(getSkinName(),"Main2 DUI Window Height",main2Height);
	
	setPrivateInt(getSkinName(),MAINID+" X",duiLayout.getLeft());
	setPrivateInt(getSkinName(),MAINID+" Y",duiLayout.getTop());
	setPrivateInt(getSkinName(),MAINID+" W",duiLayout.getWidth());
	setPrivateInt(getSkinName(),MAINID+" H",duiLayout.getHeight());
	
	delete onLoadTimer;
}

onReceiveDUICommand(string winName, string command, string param) {
  windowName = strlower(winName);

  string param1 = strlower(getToken(param,",",0));
  string param2 = strlower(getToken(param,",",1));
  string param3 = strlower(getToken(param,",",2));
  string param4 = strlower(getToken(param,",",3));
  string param5 = strlower(getToken(param,",",4));
  string param6 = strlower(getToken(param,",",5));
  string param7 = strlower(getToken(param,",",6));

  if (windowName!=MAINID) {
	  
    if (command == OPEN_DUI_REQUEST ) {
	  if (duiLayout.isVisible()) {
        closeDUIWindowContent(param1, windowName);
        openDUIWindow(param1, windowName, stringToInteger(param2), stringToInteger(param3), stringToInteger(param4), stringToInteger(param5), 
      				stringToInteger(param6), stringToInteger(param7));
  	  }
      sendDUICommand(MAINID, OPEN_DUI_GRANTED, windowName+","+getMainDUIWin(param1));
      
    } else if (command == DETACH_WINDOW_REQUEST) {
      closeDUIWindow(param1);
      sendDUICommand(MAINID, DETACH_WINDOW_GRANTED, windowName);
		
    } else if (command == CLOSE_DUI_WINDOW) {

      ConfigAttribute duiwinAttrib = getDUIwinAttrib(param1);
      if (duiwinAttrib) {
	    string attribdata = strlower(duiwinAttrib.getData());
		string winName = strlower(windowName);

        if ((attribdata == winName) || (winName == "all")) {
          closeDUIWindow(param1);
        }
      } else closeDUIWindow(param1);
      
    } else if (command == ADD_TO_DUI_LIST) {
      if ((strsearch(windowList,winName) < 0) || (windowList == "")) {

	    if (windowList == "") {
	      windowList = winName;
    	} else {
	      windowList = windowList + "," + winName;
        }
	    setPrivateString(getSkinName(), "DUI Window List", windowList);
	    initDUIAttrib(winName);
      }
    } else if (command == SET_FOCUS) {
		if (param1 == "all") param1 = strlower(MAINID);
		if (param1 == strlower(MAINID)) {
			localFocus = TRUE;
			duiLayout.setFocus();
		}
    } else if (command == REQUEST_RESIZE) {
	    resizeDUIWindow(param1, stringToInteger(param2), stringToInteger(param3));
    }
  }
}

initDUIAttrib(string windowName) {
	duiAttrib = custom_page_windows.newAttribute("Attach "+windowName+" window", "1");
}

ConfigAttribute getDUIwinAttrib(string duiWinID) {
  if (duiWinID == DUI_WIN_MAIN) {
    return dui_win_main_content_attrib;
  } else if (duiWinID == DUI_WIN_MAIN2) {
    return dui_win_main2_content_attrib;
  } else if (duiWinID == DUI_WIN_SIDE) {
    return dui_win_side_content_attrib;
  } else if (duiWinID == DUI_WIN_SIDE2) {
    return dui_win_side2_content_attrib;
  }
}

string getMainDUIWin(string duiWinID) {
  if (duiWinID == DUI_WIN_MAIN) {
    return MAINID+","+duiLayout.getID()+","+duiWindowGroupMain.getID()+",base.hole";
  } else if (duiWinID == DUI_WIN_MAIN2) {
    return MAINID+","+duiLayout.getID()+","+duiWindowGroupMain2.getID()+",base.hole";
  } else if (duiWinID == DUI_WIN_SIDE) {
    return MAINID+","+duiLayout.getID()+","+duiWindowGroupSide.getID()+",base.hole";
  } else if (duiWinID == DUI_WIN_SIDE2) {
    return MAINID+","+duiLayout.getID()+","+duiWindowGroupSide2.getID()+",base.hole";
  }
}

openDUIWindow(string duiWinID, string windowName, int min_w, int min_h, int max_w, int max_h, int def_w, int def_h) {
  if (!duiLayout.isVisible()) return;

  if (windows_hideshowonresize_attrib.getData()=="1") {
	  if (dui_onresizehide_attrib.getData()=="1") dui_onresizehide_attrib.setData("0");
  }


  boolean firstDUIOpen = !duiWindowMain.isVisible() && !duiWindowSide.isVisible(); // checks if no dui window is open yet.
  
  if (duiWinID == DUI_WIN_MAIN) {
  
    if (min_w <= 80) main_min_w = 80; else main_min_w = min_w;
    if (min_h <= 80) main_min_h = 80; else main_min_h = min_h;
    if (max_w < min_w) main_max_w = 2147483647; else main_max_w = max_w;
    if (max_h < min_h) main_max_h = 2147483647; else main_max_h = max_h;
    
    localDUIOpenClose = TRUE;
    
    duiWindowGroupMain.show();
    duiWindowMain.show();
    
    localDUIOpenClose = FALSE;
    
  } else if (duiWinID == DUI_WIN_MAIN2) {
  
    if (min_w <= 80) main2_min_w = 80; else main2_min_w = min_w;
    if (min_h <= 80) main2_min_h = 80; else main2_min_h = min_h;
    if (max_w < min_w) main2_max_w = 2147483647; else main2_max_w = max_w;
    if (max_h < min_h) main2_max_h = 2147483647; else main2_max_h = max_h;
    
    localDUIOpenClose = TRUE;
    
    duiWindowGroupMain2.show();
    duiWindowMain.show();
    
    localDUIOpenClose = FALSE;
    
   } else if (duiWinID == DUI_WIN_SIDE) {
    if (min_w <= 80) side_min_w = 80; else side_min_w = min_w;
    if (min_h <= 80) side_min_h = 80; else side_min_h = min_h;
    if (max_w < min_w) side_max_w = 2147483647; else side_max_w = max_w;
    if (max_h < min_h) side_max_h = 2147483647; else side_max_h = max_h;
    
    localDUIOpenClose = TRUE;
	  
    duiWindowGroupSide.show();
    duiWindowSide.show();
    
    localDUIOpenClose = FALSE;
    
 
  } else if (duiWinID == DUI_WIN_SIDE2) {
    if (min_w <= 80) side2_min_w = 80; else side2_min_w = min_w;
    if (min_h <= 80) side2_min_h = 80; else side2_min_h = min_h;
    if (max_w < min_w) side2_max_w = 2147483647; else side2_max_w = max_w;
    if (max_h < min_h) side2_max_h = 2147483647; else side2_max_h = max_h;
    
    localDUIOpenClose = TRUE;
	  
    duiWindowGroupSide2.show();
    duiWindowSide.show();
    
    localDUIOpenClose = FALSE;
    
  }
  
  ConfigAttribute duiwinAttrib = getDUIwinAttrib(duiWinID);
  if (duiwinAttrib) duiwinAttrib.setData(windowName);
  
  if (firstDUIOpen) { // if this is first dui window opened, set to proper/old height
    int oldh = getPrivateInt(getSkinName(),"DUI Last H",def_h+duiLayout.getHeight());
    localResize = TRUE;
    duiLayout.resize(duiLayout.getLeft(),duiLayout.getTop(),duiLayout.getWidth(), oldh);
    localResize = FALSE;
  }
     
  refreshDUILayout();
  

}

closeDUIWindow(string duiWinID) {
	localDUIOpenClose = TRUE;
	
	if (duiWinID == DUI_WIN_MAIN) {
		duiWindowGroupMain.hide();
		main_min_w = 50; main_min_h = 50; main_max_w = 2000; main_max_h = 2000;
	} else if (duiWinID == DUI_WIN_MAIN2) {
		duiWindowGroupMain2.hide();
		main2_min_w = 50; main2_min_h = 50; main2_max_w = 2000; main2_max_h = 2000;
	} else if (duiWinID == DUI_WIN_SIDE) {
		duiWindowGroupSide.hide();
		side_min_w = 50; side_min_h = 50; side_max_w = 2000; side_max_h = 2000;
	} else if (duiWinID == DUI_WIN_SIDE2) {
		duiWindowGroupSide2.hide();
		side2_min_w = 50; side2_min_h = 50; side2_max_w = 2000; side2_max_h = 2000;
	}
	
	if (!duiWindowGroupMain.isVisible() && !duiWindowGroupMain2.isVisible()) duiWindowMain.hide();
	if (!duiWindowGroupSide.isVisible() && !duiWindowGroupSide2.isVisible()) duiWindowSide.hide();
	
	localDUIOpenClose = FALSE;
	ConfigAttribute duiwinAttrib = getDUIwinAttrib(duiWinID);
	if (duiwinAttrib) duiwinAttrib.setData("");
	
	refreshDUILayout();
}


closeDUIWindowContent(string duiWinID, string windowName) {
  ConfigAttribute duiwinAttrib = getDUIwinAttrib(duiWinID);
  
  if (duiwinAttrib) {

    if (duiwinAttrib.getData()=="") return;
	  
	string attribdata = strlower(duiwinAttrib.getData());
	string winName = strlower(windowName);
	
    if (attribdata != winName) {
       setPrivateString(getSkinName(),"onDUIClosing",duiWinID);
       sendDUIWindowName = MAINID;
       sendDUICommandParam = CLOSE_DUI_WINDOW;
       sendDUIParam = duiwinAttrib.getData();
       sendDUIDelay.start();

    }
  }
}

duiLayout.onSetVisible(boolean on) {
	dui_temp_detach_attrib.setData(integerToString(on==0));
}

// has to code still :D
duiWindowMain.onSetVisible(boolean on) {
  //refreshDUILayout();
  if (localDUIOpenClose) return;
  if (on) {
	return;
  } else {
	return;
  }
}

duiWindowSide.onSetVisible(boolean on) {
  //refreshDUILayout();
  if (localDUIOpenClose) return;
  if (on) {
	return;
  } else {
	return;
  }
}

sendDUIDelay.onTimer() {
  stop();
  setPrivateString(getSkinName(),"onDUIClosing","");
  sendDUICommand(sendDUIWindowName, sendDUICommandParam, sendDUIParam);
}


// script to force-resize DUI to a diff width/height - mostly used for Video window (1x, 2x etc.)
resizeDUIWindow(string duiWinID, int neww, int newh) {
	int newLayW = duiLayout.getWidth();
	int newLayH = duiLayout.getHeight();
	int minWidth = getMinWidth();
	int minHeight = getMinHeight();

	int mainWidth = getMainWidth();
	
	if (duiWinID == DUI_WIN_MAIN) {
		newLayW = newLayW - duiWindowGroupMain.getWidth() + neww;
		newLayH = newLayH - duiWindowGroupMain.getHeight() + newh;
		
		int newHeightDiff = duiLayout.getHeight() - newLayH;
		
		duiLayout.resize(duiLayout.getLeft(), duiLayout.getTop() + newHeightDiff, newLayW, newLayH);
	} else if (duiWinID == DUI_WIN_MAIN2) {
	
		newLayW = neww;
		if (duiWindowSide.isVisible()) newLayW = newLayW + resizerHorizGroup.getWidth() + sideWidth;
		newLayW = newLayW + duiLayout.getWidth() - frameGroup.getWidth();
		
		main2Height = newH;
		int newLayH = main2Height;
		if (duiWindowGroupMain.isVisible()) newLayH = newLayH + duiWindowGroupMain.getHeight() + resizerMainGroup.getHeight();
		newLayH = newLayH + duiLayout.getHeight() - frameGroup.getHeight();
		
		int newHeightDiff = duiLayout.getHeight() - newLayH;
		
		duiLayout.resize(duiLayout.getLeft(), duiLayout.getTop() + newHeightDiff, newLayW, newLayH);
	} else if (duiWinID == DUI_WIN_SIDE) {
		sideWidth = neww;
		newLayW = sideWidth;
		if (duiWindowMain.isVisible()) newLayW = newLayW + mainWidth + resizerHorizGroup.getWidth();
		newLayW = newLayW + duiLayout.getWidth() - frameGroup.getWidth();
		
		newLayH = newLayH - duiWindowGroupSide.getHeight() + newh;
		
		int newHeightDiff = duiLayout.getHeight() - newLayH;
		
		duiLayout.resize(duiLayout.getLeft(), duiLayout.getTop() + newHeightDiff, newLayW, newLayH);
	} else if (duiWinID == DUI_WIN_SIDE2) {
		sideWidth = neww;
		newLayW = sideWidth;
		if (duiWindowMain.isVisible()) newLayW = newLayW + mainWidth + resizerHorizGroup.getWidth();
		newLayW = newLayW + duiLayout.getWidth() - frameGroup.getWidth();
		
		side2Height = newH;
		newLayH = newLayH - duiWindowGroupSide2.getHeight() + side2Height;
		
		int newHeightDiff = duiLayout.getHeight() - newLayH;
		
		duiLayout.resize(duiLayout.getLeft(), duiLayout.getTop() + newHeightDiff, newLayW, newLayH);
	}
}

duiWindowMain.onResize(int wx, int wy, int ww, int wh) {
  //autoResizeDUIWindow(duiWindowMain, duiWindowGroupMain, main_min_w, main_min_h, main_max_w, main_max_h);
  return;
}

duiLayout.onResize(int wx, int wy, int ww, int wh) {
	if (windows_hideshowonresize_attrib.getData()=="0") return;
	if (localResize) { return; }
	if (dui_on_transition_attrib.getData()=="1") return;
	if (onLoadTimer.isRunning()) return;

	int minh = minLayoutH;
	int minw = minLayoutW;
	if (minh == stringToInteger(duiLay_min_h)) minh += 1; // if no windows are open, dont on resize show
	
	string resizeAttrib = dui_onresizehide_attrib.getData();
	if (resizeAttrib=="0") {
		if (wh < minh) {
			dui_onresizehide_attrib.setData("1");
			resizeTimer.start();
		} else if (wh == minh) {
			duiLayout.setXMLParam("minimum_h",duiLay_min_h);
			
			if (wh < minh) wh = minh;
			if (ww < minw) ww = minw;
			resizeNewH = wh;
			resizeNewW = ww;
			resizeTimer.start();
		}
	} else {
		if (wh > stringToInteger(duiLay_min_h)) {
			if (wh < minh) wh = minh;
			if (ww < minw) ww = minw;
			dui_onresizehide_attrib.setData("0");
			resizeNewH = wh;
			resizeNewW = ww;
			resizeTimer.start();
		}
	}
}

resizeTimer.onTimer() {
	stop();
	localResize = TRUE;
	
	if (dui_onresizehide_attrib.getData()=="1") {
		duiLayout.setXMLParam("minimum_w",duiLay_min_w);
		duiLayout.resize(duiLayout.getLeft(), duiLayout.getTop(), duiLayout.getWidth(), stringToInteger(duiLay_min_h));
	} else {
		duiLayout.setXMLParam("minimum_w",integerToString(minLayoutW));
		duiLayout.resize(duiLayout.getLeft(), duiLayout.getTop(), resizeNewW, resizeNewH);
	}
	
	localResize = FALSE;
}

windows_hideshowonresize_attrib.onDataChanged() {
	if (dui_onresizehide_attrib.getData()=="1") dui_onresizehide_attrib.setData("0");
	
	refreshDUILayout();
}

frameGroup.onResize(int wx, int wy, int ww, int wh) {
	if (localResize) { return; }
	
	refreshDUILayout();
	
	duiLayout.onMove();
}

duiLayout.onScale(double newscale) {
	duiLayout.onMove();
}

// toggles every 100 msec
onGenTimer100ms() {
	if (localFocus) { localFocus = FALSE; return; }

	dui_on_focus_attrib.setData("0");
	
	dui_checkfocus_attrib.setData(integerToString(dui_checkfocus_attrib.getData()=="0"));
	
	boolean duiFocus = duiLayout.isActive() || (dui_on_focus_attrib.getData() == "1");

	if ((lastFocus == FALSE) && (duiFocus)) {
		sendDUICommand(MAINID, SET_FOCUS, "all");
		duiLayout.setFocus();	
	}
	lastFocus = duiFocus;
}

/*** computes all the proper dimensions of DUI windows, also sets min width and height of main layout ***/
refreshDUILayout() {
	// disables all window resizing until all groups have been properly resized
	dui_on_transition_attrib.setData("1"); 
	localResize = TRUE;
	
	int main_guix = stringToInteger(duiWindowMain.getXMLParam("x"));
	int main_guiy = stringToInteger(duiWindowMain.getXMLParam("y"));
	int main_guiw = stringToInteger(duiWindowMain.getXMLParam("w"));
	int main_guih = stringToInteger(duiWindowMain.getXMLParam("h"));
	
	// resets duiLayout to proper dimensions
	int duiWidth = duiLayout.getWidth();
	int duiHeight = duiLayout.getHeight();
	int minWidth = getMinWidth();
	int minHeight = getMinHeight();
	
	if (duiWidth < minWidth) duiWidth = minWidth;
	if (duiHeight < minHeight) duiHeight = minHeight;
	
	if (windows_hideshowonresize_attrib.getData()=="0") {
		duiLayout.setXMLParam("minimum_w",integerToString(minWidth));
		duiLayout.setXMLParam("minimum_h",integerToString(minHeight));
	} else {
		duiLayout.setXMLParam("minimum_w",integerToString(minWidth));
		duiLayout.setXMLParam("minimum_h",duiLay_min_h);
	}
	
	minLayoutH = minHeight;	
	minLayoutW = minWidth;	
	if (!duiWindowMain.isVisible() && !duiWindowSide.isVisible()) {
	  duiLayout.setXMLParam("maximum_w",duiLay_max_w);
	  duiLayout.setXMLParam("maximum_h",integerToString(minHeight));
	  
	  //int newHeightDiff = duiLayout.getHeight() - minHeight;
	  
	  //duiLayout.resize(duiLayout.getLeft(),duiLayout.getTop() + newHeightDiff, duiWidth, minHeight);
	  duiLayout.resize(duiLayout.getLeft(),duiLayout.getTop(), duiWidth, minHeight);
	} else {
	  duiLayout.setXMLParam("maximum_w",duiLay_max_w);
	  duiLayout.setXMLParam("maximum_h",duiLay_max_h);

	  //int newHeightDiff = duiLayout.getHeight() - duiHeight;
	  //if ((duiLayout.getTop() + newHeightDiff) < getViewPortTop()) newHeightDiff = getViewPortTop()-duiLayout.getTop();
	  
	  //duiLayout.resize(duiLayout.getLeft(),duiLayout.getTop() + newHeightDiff, duiWidth, duiHeight);
	  duiLayout.resize(duiLayout.getLeft(),duiLayout.getTop(), duiWidth, duiHeight);
	  
	  if (duiLayout.isVisible()) setPrivateInt(getSkinName(),"DUI Last H",duiLayout.getHeight()); // used for height restore
	}

	int mainWidth = getMainWidth();
	int main_min_width_computed = getMainMinWidth();
	int side_min_width_computed = getSideMinWidth();

	if (sideWidth < side_min_width_computed) sideWidth = side_min_width_computed;
	if (mainWidth <= main_min_width_computed && duiWindowMain.isVisible() && duiWindowSide.isVisible()) 
		sideWidth = framegroup.getWidth() - main_min_width_computed - resizerHorizGroup.getWidth();
	
	if (duiWindowMain.isVisible() && duiWindowSide.isVisible()) {
		setGuiCoord(duiWindowMain,"0,0,"+integerToString(-sideWidth-resizerHorizGroup.getWidth())+",0,0,0,1,1");
		setGuiCoord(duiWindowSide,integerToString(-sideWidth)+",0,"+integerToString(sideWidth)+",0,1,0,0,1");
		resizerHorizGroup.show();
		setGuiCoord(resizerHorizGroup,integerToString(-sideWidth-resizerHorizGroup.getWidth())+",0,"+
										integerToString(resizerHorizGroup.getWidth())+",0,1,0,0,1");
	} else if (duiWindowMain.isVisible() && !duiWindowSide.isVisible()) {
		setGuiCoord(duiWindowMain,"0,0,0,0,0,0,1,1");
		resizerHorizGroup.hide();
	} else if (!duiWindowMain.isVisible() && duiWindowSide.isVisible()) {
		setGuiCoord(duiWindowSide,"0,0,0,0,0,0,1,1");
		resizerHorizGroup.hide();
	}
	
	
	// computations and adjustments of left DUI Windows (lefttop and leftbottom)
	int main_min_height_computed = getMainMinHeight();
	int main1Height = duiWindowMain.getHeight() - main2Height - resizerMainGroup.getHeight();
	
	if (main2Height < main2_min_h) main2Height = main2_min_h;
	if (main1Height < main_min_h && duiWindowGroupMain.isVisible() && duiWindowGroupMain2.isVisible()) 
		main2Height = duiWindowMain.getHeight() - resizerMainGroup.getHeight() - main_min_h;
	
	if (duiWindowGroupMain.isVisible() && duiWindowGroupMain2.isVisible()) {
		setGuiCoord(duiWindowGroupMain,"0,0,0,"+integerToString(-main2Height-resizerMainGroup.getHeight())+",0,0,1,1");
		setGuiCoord(duiWindowGroupMain2,"0,"+integerToString(-main2Height)+",0,"+
					integerToString(main2Height)+",0,1,1,0");
		resizerMainGroup.show();
		setGuiCoord(resizerMainGroup,"0,"+integerToString(-main2Height-resizerMainGroup.getHeight())+",0,"+
				integerToString(resizerMainGroup.getHeight())+",0,1,1,0");
	} else if (duiWindowGroupMain.isVisible() && !duiWindowGroupMain2.isVisible()) {
		setGuiCoord(duiWindowGroupMain,"0,0,0,0,0,0,1,1");
		resizerMainGroup.hide();
	} else if (!duiWindowGroupMain.isVisible() && duiWindowGroupMain2.isVisible()) {
		setGuiCoord(duiWindowGroupMain2,"0,0,0,0,0,0,1,1");
		resizerMainGroup.hide();
	}
	
	// computations and adjustments of right DUI Windows (righttop and rightbottom)
	int side_min_height_computed = getSideMinHeight();
	int side1Height = duiWindowSide.getHeight() - side2Height - resizerSideGroup.getHeight();
	
	if (side2Height < side2_min_h) side2Height = side2_min_h;
	if (side1Height < side_min_h && duiWindowGroupSide.isVisible() && duiWindowGroupSide2.isVisible()) 
		side2Height = duiWindowSide.getHeight() - resizerSideGroup.getHeight() - side_min_h;

	if (duiWindowGroupSide.isVisible() && duiWindowGroupSide2.isVisible()) {
		setGuiCoord(duiWindowGroupSide,"0,"+integerToString(side2Height+resizerSideGroup.getHeight())+",0,"+
					integerToString(-side2Height-resizerSideGroup.getHeight())+",0,0,1,1");
		setGuiCoord(duiWindowGroupSide2,"0,0,0,"+
					integerToString(side2Height)+",0,0,1,0");
		resizerSideGroup.show();
		setGuiCoord(resizerSideGroup,"0,"+integerToString(side2Height)+",0,"+
				integerToString(resizerSideGroup.getHeight())+",0,0,1,0");
	} else if (duiWindowGroupSide.isVisible() && !duiWindowGroupSide2.isVisible()) {
		setGuiCoord(duiWindowGroupSide,"0,0,0,0,0,0,1,1");
		resizerSideGroup.hide();
	} else if (!duiWindowGroupSide.isVisible() && duiWindowGroupSide2.isVisible()) {
		setGuiCoord(duiWindowGroupSide2,"0,0,0,0,0,0,1,1");
		resizerSideGroup.hide();
	}
	
	localResize = FALSE;
	dui_on_transition_attrib.setData("0"); // enable back..
}

resizerHorizHandle.onLeftButtonDown(int x, int y) {
  resizerDown = TRUE;
  setXMLParam("resize","");
  duiLayout.setXMLParam("minimum_w",integerToString(duiLayout.getWidth()));
  duiLayout.setXMLParam("minimum_h",integerToString(duiLayout.getHeight()));
  duiLayout.setXMLParam("maximum_w",integerToString(duiLayout.getWidth()));
  duiLayout.setXMLParam("maximum_h",integerToString(duiLayout.getHeight()));
}

resizerHorizHandle.onMouseMove(int x, int y) {
  if (!resizerDown) return;
  sideWidth = (frameGroup.getLeft()+frameGroup.getWidth())-x;
  refreshDUILayout();
}


resizerHorizHandle.onLeftButtonUp(int x, int y) {
  resizerDown = FALSE;

  refreshDUILayout();
}

resizerMainHandle.onLeftButtonDown(int x, int y) {
  resizerDown = TRUE;

}

resizerMainHandle.onMouseMove(int x, int y) {
  if (!resizerDown) return;
  main2Height = frameGroup.getHeight() - y + frameGroup.getTop();
  refreshDUILayout();
}


resizerMainHandle.onLeftButtonUp(int x, int y) {
  resizerDown = FALSE;

  refreshDUILayout();
}

resizerSideHandle.onLeftButtonDown(int x, int y) {
  resizerDown = TRUE;
}

resizerSideHandle.onMouseMove(int x, int y) {
  if (!resizerDown) return;
  side2Height = y - frameGroup.getTop();
  refreshDUILayout();
}


resizerSideHandle.onLeftButtonUp(int x, int y) {
  resizerDown = FALSE;

  refreshDUILayout();
}


// computes for current dui_main window width
int getMainWidth() {
	if (duiWindowMain.isVisible())
		return frameGroup.getWidth()-sideWidth-resizerHorizGroup.getWidth();
	else
		return 0;
}

// computes for the minimum layout width
int getMinWidth() {
  int minw = 0;
  int mainminw = getMainMinWidth();
  int sideminw = getSideMinWidth();
  
  if (duiWindowMain.isVisible()) minw = minw + mainminw;
  if (duiWindowSide.isVisible()) minw = minw + sideminw;

  if (duiWindowMain.isVisible() && duiWindowSide.isVisible()) minw = minw + resizerHorizGroup.getWidth();
	
  minw = (duiLayout.getWidth()-frameGroup.getWidth()) + minw;
  if (minw < stringToInteger(duiLay_min_w)) minw = stringToInteger(duiLay_min_w);
  
  return minw;
}

// computes for the minimum layout height
int getMinHeight() {
  int minh = 0;
  int mainminh = getMainMinHeight();
  int sideminh = getSideMinHeight();
  
  if (duiWindowMain.isVisible() && (minh <  mainminh)) minh = mainminh;
  if (duiWindowSide.isVisible() && (minh <  sideminh)) minh = sideminh;
  
  if (minh > 0) minh = (duiLayout.getHeight()-frameGroup.getHeight()) + minh;
  
  if (minh < stringToInteger(duiLay_min_h)) minh = stringToInteger(duiLay_min_h);

  return minh;
}

// computes for the minimum width of main & main2 windows
int getMainMinWidth() {
	int minwidth = 0;
	
	if (duiWindowGroupMain.isVisible()) minwidth = main_min_w + duiHoleToGroupDiffWidth;
	if ((duiWindowGroupMain2.isVisible()) && (minwidth < (main2_min_w + duiHoleToGroupDiffWidth)))
		minwidth = main2_min_w + duiHoleToGroupDiffWidth;
	
	return minwidth;
}

// computes for the minimum height of main & main2 windows
int getMainMinHeight() {
	int minheight = 0;
	
	if (duiWindowGroupMain.isVisible()) minheight = minheight + main_min_h + duiHoleToGroupDiffHeight;
	if (duiWindowGroupMain2.isVisible()) minheight = minheight + main2_min_h + duiHoleToGroupDiffHeight;
	
	if (duiWindowGroupMain.isVisible() && duiWindowGroupMain2.isVisible()) minheight += resizerMainGroup.getHeight();
	
	return minheight;
}

// computes for the minimum width of side & side2 windows
int getSideMinWidth() {
	int minwidth = 0;
	
	if (duiWindowGroupSide.isVisible()) minwidth = minwidth + side_min_w + duiHoleToGroupDiffWidth;
	if ((duiWindowGroupSide2.isVisible()) && (minwidth < (side2_min_w + duiHoleToGroupDiffWidth)))
		minwidth = side2_min_w + duiHoleToGroupDiffWidth;
	
	return minwidth;
}

// computes for the minimum height of side & side2 windows
int getSideMinHeight() {
	int minheight = 0;
	
	if (duiWindowGroupSide.isVisible()) minheight = minheight + side_min_h + duiHoleToGroupDiffHeight;
	if (duiWindowGroupSide2.isVisible()) minheight = minheight + side2_min_h + duiHoleToGroupDiffHeight;
	
	if (duiWindowGroupSide.isVisible() && duiWindowGroupSide2.isVisible()) minheight += resizerSideGroup.getHeight();
	
	return minheight;
}