/***

KameleonDUI - DUI-Standardframe.m
by Leechbite

standardframe script for the DUI interface.

***/

#include <lib/std.mi>
#include "attribs.m"
#include "utilities.m"
#include "dui-utilities.m"

function setNewGroup(String groupid);
function setToDUIMode();
function resetFromDUIMode();
function syncFrame();

function toggleComponent(string compID);

Global Group frameGroup, content, titlebar;
Global layout frameLayout;
Global Container frameContainer;
Global String x, y, w, h, rx, ry, rw, rh;
Global String min_w, min_h, max_w, max_h;
Global Layer mouselayer;
Global Button Sysmenu;

global container duiMainContainer;
global layout duiMainLayout;
global group duiMainGroup;
global guiObject duiMainWin;
global timer delaySync, delayReattach;

Global String duix, duiy, duiw, duih;
Global String duimin_w, duimin_h, duimax_w, duimax_h;

global text titleText;
global ConfigAttribute currwindow_attach_attrib, currwindow_duiwindow_attrib, currwindow_tempdetach_attrib;
global guiObject concealer, duiFrame, wasabiFrame;
global string duiWindow; // determines to which DUI-window the skin shows. set from param=""
global string windowName;
global boolean localClose = FALSE;
global boolean localFocus = FALSE;
global boolean lastFocus = FALSE;
global boolean localResize = FALSE;
global boolean noClose = FALSE;
global timer noCloseTimer;

global boolean allLoaded = FALSE;
global boolean onLoad = TRUE, onUnload = FALSE;
global boolean mainMove = FALSE;

global button attachButton;
global button detachButton;
global button windowButton;

global int diffWidthDUIWasabi;  // both stores the difference in width (& height) of DUI and Wasabi frames
global int diffHeightDUIWasabi;

global timer onStartTimer, syncTimer;

System.onScriptLoaded() {
	initAttribs();
	
	frameGroup = getScriptGroup();
	frameLayout = frameGroup.getParentLayout();
	frameContainer = frameLayout.getContainer();
	String param = getParam();
	x = getToken(param, ",", 0);
	y = getToken(param, ",", 1);
	w = getToken(param, ",", 2);
	h = getToken(param, ",", 3);
	rx = getToken(param, ",", 4);
	ry = getToken(param, ",", 5);
	rw = getToken(param, ",", 6);
	rh = getToken(param, ",", 7);
	
	min_w = frameLayout.getXMLParam("minimum_w");
	min_h = frameLayout.getXMLParam("minimum_h");
	max_w = frameLayout.getXMLParam("maximum_w");
	max_h = frameLayout.getXMLParam("maximum_h");
	
	group temp = frameGroup.findObject("button.attach"); 	// modify this portion for it to get the proper attach &
	attachButton = temp.findObject("xuibutton");			// detach buttons.
	detachButton = frameGroup.findObject("dui.detach");
	windowButton = frameGroup.findObject("dui.window");
	
	duiWindow = strlower(getToken(param, ",", 8));
	sysmenu = frameGroup.findObject("sysmenu");
	duiFrame = frameGroup.findObject(getToken(param, ",", 13));
	concealer = frameLayout.findObject("dui.concealer");
	wasabiFrame = frameGroup.findObject("wasabi.frame.layout");
	concealer.show();
	duiFrame.hide();
	
	duix = getToken(param, ",", 9);
	duiy = getToken(param, ",", 10);
	duiw = getToken(param, ",", 11);
	duih = getToken(param, ",", 12);
	
	
	diffWidthDUIWasabi = stringToInteger(w) - stringToInteger(duiw); // width difference between standard layout and dui
	duimin_w = integerToString(stringToInteger(min_w) + diffWidthDUIWasabi);
	duimax_w = integerToString(stringToInteger(max_w) + diffWidthDUIWasabi);
	
	diffHeightDUIWasabi = stringToInteger(h) - stringToInteger(duih); // height difference between standard layout and dui
	duimin_h = integerToString(stringToInteger(min_h) + diffHeightDUIWasabi);
	duimax_h = integerToString(stringToInteger(max_h) + diffHeightDUIWasabi);
	
	delaySync = new Timer;
	delaySync.setDelay(1);
	
	delayReattach = new Timer;
	delayReattach.setDelay(1);
	
	titleText = frameGroup.findObject("frame.title.dummy");
	
	onStartTimer = new Timer;
	onStartTimer.setDelay(100);
	onStartTimer.start();
	
	syncTimer = new Timer;
	syncTimer.setDelay(100);
	if (frameLayout.isVisible()) syncTimer.start();
	
	noCloseTimer = new Timer;
	noCloseTimer.setDelay(200);
}

// **** DUI scripts
system.onScriptUnloading() {
	onUnLoad = TRUE;
	
	duiMainContainer = NULL;
	duiMainLayout = NULL;
	duiMainGroup = NULL;
	duiMainWin = NULL;
	
	if ((currwindow_attach_attrib != NULL) && (!localClose)) {
		if (currwindow_attach_attrib.getData()=="1") {
			sendDUICommand(windowName, CLOSE_DUI_WINDOW, duiWindow);
		} else {
			setPrivateInt(getSkinName(),strupper(windowName)+"Last X", frameLayout.getLeft());
			setPrivateInt(getSkinName(),strupper(windowName)+"Last Y", frameLayout.getTop());
			setPrivateInt(getSkinName(),strupper(windowName)+"Last W", frameLayout.getWidth());
			setPrivateInt(getSkinName(),strupper(windowName)+"Last H", frameLayout.getHeight());
		}
	}
	
	delete onStartTimer;
	delete delaySync;	
	delete delayReattach;
	delete syncTimer;
	delete noCloseTimer;
}

frameLayout.onSetVisible(boolean on) {
	if (on)	syncTimer.start(); else syncTimer.stop();
	
	if (onStartTimer.isRunning()) return;
	if (!on) {
		duiMainContainer = NULL;
		duiMainLayout = NULL;
		duiMainGroup = NULL;
		duiMainWin = NULL;
		allLoaded = FALSE;
		if ((currwindow_attach_attrib != NULL) && (!localClose)) {
			if (currwindow_attach_attrib.getData()=="1") sendDUICommand(windowName, CLOSE_DUI_WINDOW, duiWindow);
			else {
				setPrivateInt(getSkinName(),strupper(windowName)+"Last X", frameLayout.getLeft());
				setPrivateInt(getSkinName(),strupper(windowName)+"Last Y", frameLayout.getTop());
				setPrivateInt(getSkinName(),strupper(windowName)+"Last W", frameLayout.getWidth());
				setPrivateInt(getSkinName(),strupper(windowName)+"Last H", frameLayout.getHeight());
			}
		}
	} else { 
		if (strleft(titleText.getText(),1)!=":") onStartTimer.onTimer();
	}
}

onStartTimer.onTimer() {
	string newTitle = titleText.getText();
	if (strleft(newTitle,1)!=":") {
		if (onStartTimer.isRunning()) onStartTimer.stop();
		if (!frameLayout.isVisible()) return;
		newTitle = getToken(newTitle,"\t",0);
		sendDUICommand(newTitle, ADD_TO_DUI_LIST, "");
		currwindow_attach_attrib = custom_page_windows.newAttribute("Attach "+newTitle+" window", "1");
		currwindow_duiwindow_attrib = custom_page_nonexposed.newAttribute("DUI window "+newTitle, duiWindow);
		currwindow_tempdetach_attrib = custom_page_nonexposed.newAttribute("Temp Detach "+newTitle+" window", "0");
		duiWindow = currwindow_duiwindow_attrib.getData();
		currwindow_tempdetach_attrib.onDataChanged();
		
		windowName = newTitle;
		if ((currwindow_attach_attrib.getData()=="1") || (currwindow_tempdetach_attrib.getData()=="1")) {
			concealer.show();
		  	if (content)
		  		sendDUICommand(windowName, OPEN_DUI_REQUEST, duiWindow+","+duimin_w+","+duimin_h+","+duimax_w+","+duimax_h+","+
							integerToString(frameGroup.getWidth())+","+
							integerToString(frameGroup.getHeight()));
			else
				sendDUICommand(windowName, OPEN_DUI_REQUEST, duiWindow+","+duimin_w+","+duimin_h+","+duimax_w+","+duimax_h+","+
							duimin_w+","+duimin_h);
		} else {
			resetFromDUIMode();
			concealer.hide();
			frameLayout.setFocus();
		}
		
		allLoaded = TRUE;
	}
}

currwindow_attach_attrib.onDataChanged() {
	if (getData()=="0") {
		syncTimer.stop();
		concealer.hide();
		if (frameLayout.isVisible()) {
			string nextcomp = getPrivateString(getSkinName(),strupper(windowName)+" Next Window","");
			if ((windows_duitab_attrib.getData()=="1") && (nextcomp!="") && (currwindow_tempdetach_attrib.getData()!="1")) {
				resetFromDUIMode();
		    	concealer.hide();
				
				toggleComponent(nextcomp);
			
				frameLayout.setFocus();

			} else
				sendDUICommand(windowName, DETACH_WINDOW_REQUEST, duiWindow);
		}
	
		int lastX = getPrivateInt(getSkinName(),strupper(windowName)+"Last X", 0); 
		int lastY = getPrivateInt(getSkinName(),strupper(windowName)+"Last Y", 0); 
		int lastW = getPrivateInt(getSkinName(),strupper(windowName)+"Last W", stringToInteger(min_w)); 
		int lastH = getPrivateInt(getSkinName(),strupper(windowName)+"Last H", stringToInteger(min_h));
		
		// double checks the dimension, makes sure its still inside limits
		if (max_w!="") if (lastW > stringToInteger(max_w)) lastW = stringToInteger(max_w);
		if (lastW < stringToInteger(min_w)) lastW = stringToInteger(min_w);
		if (max_h!="") if (lastH > stringToInteger(max_h)) lastH = stringToInteger(max_h);
		if (lastH < stringToInteger(min_h)) lastH = stringToInteger(min_h);
		
		if ((lastX + lastW) < 0) lastX = 0;
		if ((lastY + lastH) < 0) lastY = 0;
		if (lastX > getViewPortWidth()) lastX = getViewPortWidth()-lastW;
		if (lastY > getViewPortHeight()) lastY = getViewPortHeight()-lastH;
		
		frameLayout.resize(lastX, lastY, lastW, lastH);

	} else {
		currwindow_tempdetach_attrib.setData("0");
		
		setPrivateInt(getSkinName(),strupper(windowName)+"Last X", frameLayout.getLeft());
		setPrivateInt(getSkinName(),strupper(windowName)+"Last Y", frameLayout.getTop());
		setPrivateInt(getSkinName(),strupper(windowName)+"Last W", frameLayout.getWidth());
		setPrivateInt(getSkinName(),strupper(windowName)+"Last H", frameLayout.getHeight());
		concealer.hide();
		
		if (frameLayout.isVisible()) {
			if (content)
				sendDUICommand(windowName, OPEN_DUI_REQUEST, duiWindow+","+duimin_w+","+duimin_h+","+duimax_w+","+duimax_h+","+
							integerToString(content.getWidth()-stringToInteger(duiw))+","+
							integerToString(content.getHeight()-stringToInteger(duih)));
			else
				sendDUICommand(windowName, OPEN_DUI_REQUEST, duiWindow+","+duimin_w+","+duimin_h+","+duimax_w+","+duimax_h+","+
							duimin_w+","+duimin_h);
		}
		syncTimer.start();
	}
}

currwindow_tempdetach_attrib.onDataChanged() {
	if (getData()=="1") {
		attachButton.setXMLParam("ghost","1");
	} else {
		attachButton.setXMLParam("ghost","0");
	}
}

attachButton.onLeftClick() {
	if (currwindow_attach_attrib) currwindow_attach_attrib.setData("1");
}

detachButton.onLeftClick() {
	if (currwindow_attach_attrib) currwindow_attach_attrib.setData("0");
}

windowButton.onLeftClick() {
	if (!currwindow_duiwindow_attrib) return;
	
	popupMenu DUIwindowMenu;
	
	DUIwindowMenu = new popupMenu;
	DUIwindowMenu.addCommand("- Select DUI Window -", -1, 0, 1);
	DUIwindowMenu.addSeparator();
	DUIwindowMenu.addCommand(" Main", 1, currwindow_duiwindow_attrib.getData()==DUI_WIN_MAIN, 0);
	DUIwindowMenu.addCommand(" Bottom Main", 2, currwindow_duiwindow_attrib.getData()==DUI_WIN_MAIN2, 0);
	DUIwindowMenu.addCommand(" Side", 3, currwindow_duiwindow_attrib.getData()==DUI_WIN_SIDE, 0);
	DUIwindowMenu.addCommand(" Top Side", 4, currwindow_duiwindow_attrib.getData()==DUI_WIN_SIDE2, 0);
	DUIwindowMenu.addSeparator();
	DUIwindowMenu.addCommand(" Reset to default", 5, 0, 0);
	
	string lastTip = getXMLParam("tooltip");
	setXMLParam("tooltip", "");

  	int res = DUIwindowMenu.popAtMouse();
  	delete DUIwindowMenu;
  	
  	if (res == 1) currwindow_duiwindow_attrib.setData(DUI_WIN_MAIN);
  	if (res == 2) currwindow_duiwindow_attrib.setData(DUI_WIN_MAIN2);
  	if (res == 3) currwindow_duiwindow_attrib.setData(DUI_WIN_SIDE);
  	if (res == 4) currwindow_duiwindow_attrib.setData(DUI_WIN_SIDE2);
  	if (res == 5) currwindow_duiwindow_attrib.setData(duiWindow);
  	
  	setXMLParam("tooltip", lastTip);
}

currwindow_duiwindow_attrib.onDataChanged() {
	if (!currwindow_duiwindow_attrib) return;
	
	string lastDUIWindow = duiWindow;
	duiWindow = getData();

	string nextcomp = getPrivateString(getSkinName(),strupper(windowName)+" Next Window","");
	
	if (frameLayout.isVisible()) {
		if ((windows_duitab_attrib.getData()=="1") && (nextcomp!="")) {
			noClose = TRUE;
			toggleComponent(nextcomp);
			noCloseTimer.start(); //turns off onClose to FALSE
		}
		
			if (content)
				sendDUICommand(windowName, OPEN_DUI_REQUEST, duiWindow+","+duimin_w+","+duimin_h+","+duimax_w+","+duimax_h+","+
							integerToString(content.getWidth()-stringToInteger(duiw))+","+
							integerToString(content.getHeight()-stringToInteger(duih)));
			else
				sendDUICommand(windowName, OPEN_DUI_REQUEST, duiWindow+","+duimin_w+","+duimin_h+","+duimax_w+","+duimax_h+","+
							duimin_w+","+duimin_h);
	}
	
	
	if ((windows_duitab_attrib.getData()!="1") || (nextcomp==""))
		sendDUICommand(windowName, CLOSE_DUI_WINDOW, lastDUIWindow);
	//sendDUICommand(windowName, TRANSFER_DUI_WINDOW, duiWindow+","+duimin_w+","+duimin_h+","+duimax_w+","+duimax_h+","+lastDUIWindow);
}

noCloseTimer.onTimer() {
	noClose = FALSE;
	stop();
}

dui_onresizehide_attrib.onDataChanged() {
	if (getData()=="1") {
		if (frameLayout.isVisible()) system.triggerAction(frameLayout,"close","");
	}
}

onReceiveDUICommand(string winName, string command, string param) {
	string lowWinName = strlower(winName);
	string lowWinName2 = strlower(windowName);
	if (lowWinName == lowWinName2) return;
	
	string param1 = strlower(gettoken(param,",",0));
	
	if (winName == MAINID) {
	
		if (command == OPEN_DUI_GRANTED) {
		  if (currwindow_attach_attrib) { 
			  if ((currwindow_attach_attrib.getData()=="0") && (currwindow_tempdetach_attrib.getData()=="0")) return; }
		  if (param1 == "all") param1 = strlower(windowName);
		  if (param1 == strlower(windowName)) {

		    string duiContainerID = strlower(gettoken(param,",",1));
		    string duiLayoutID = strlower(gettoken(param,",",2));
		    string duiGroupID = strlower(gettoken(param,",",3));
		    string duiWinID = strlower(gettoken(param,",",4));
		    
		    duiMainContainer = getContainer(duiContainerID);
		    if (duiMainContainer) duiMainLayout = duiMainContainer.getLayout(duiLayoutID);
		    if (duiMainLayout) duiMainGroup = duiMainLayout.findObject(duiGroupID);
		    if (duiMainGroup) duiMainWin = duiMainGroup.findObject(duiWinID);
		    if (duiMainWin) duiMainWin.show();

		    
		    // if dui windows are not visible/present, detach back components
		    if (duiMainWin) {
				if (!duiMainWin.isVisible()) { 
					currwindow_tempdetach_attrib.setData("1");
					currwindow_attach_attrib.setData("0");
				} else { 
					currwindow_tempdetach_attrib.setData("0");
					if (currwindow_attach_attrib.getData()=="0") currwindow_attach_attrib.setData("1");
				}
		    } else
		    	currwindow_attach_attrib.setData("0");
		    	
   			concealer.hide();
   			if (currwindow_attach_attrib.getData()=="1") {
		    	delaySync.start();
		    	setToDUIMode();
	    	}

		    
		  }
		} else if (command == DETACH_WINDOW_GRANTED) {
		  if (param1 == "all") param1 = strlower(windowName);
		  if (param1 == strlower(windowName)) {
		    resetFromDUIMode();
		    concealer.hide();
		  }
		} else if (command == CLOSE_DUI_WINDOW) {
			if (currwindow_attach_attrib) { if (currwindow_attach_attrib.getData()=="0") return; } 
			if (param1 == "all") param1 = strlower(windowName);
			if ((param1 == strlower(windowName)) && !noClose) {
				localClose = 1;
				system.triggerAction(frameLayout,"close","");
				localClose = 0;
			}
		}
	}

  if (command == SET_FOCUS) {
	  if (currwindow_attach_attrib) { if (currwindow_attach_attrib.getData()=="0") return; } 
	  if (param1 == "all") param1 = strlower(windowName);
  	  if (param1 == strlower(windowName)) {
	    localFocus = TRUE;
	    frameLayout.setFocus();
       }
  }
}

delaySync.onTimer() {
	stop();
	if (currwindow_attach_attrib.getData()=="0") return;
	if (duiMainGroup) syncFrame();
	concealer.hide();
}


frameLayout.onResize(int wx, int wy, int ww, int wh) {
	if (localResize) return;
	if (duiMainLayout == NULL) return;
	if (currwindow_attach_attrib.getData()!="1") return;
	
	localResize = TRUE;
	if ((duiMainWin.getWidth() <= 0) || (duiMainWin.getHeight() <= 0)) return; // protects from incomplete resize of duiMainWin
	if ((ww!=duiMainWin.getWidth()) || (wh!=duiMainWin.getHeight()))
		sendDUICommand(windowName, REQUEST_RESIZE, duiWindow+","+integerToString(ww)+","+integerToString(wh));
	localResize = FALSE;
}

syncFrame() {
  if (!frameLayout.isVisible()) return;
  localResize = TRUE;
  if (duiMainWin) {
	if (concealer.isVisible()) concealer.hide();
	  
	int screenx = duiMainWin.getLeft()*duiMainLayout.getScale() + duiMainLayout.getLeft();	// clientToScreen seems to be buggy on constanly changing layouts. 
	int screeny = duiMainWin.getTop()*duiMainLayout.getScale() + duiMainLayout.getTop();		// it uses the old main layout coordinates and the new resized ones.
	int screenw = duiMainWin.getWidth();
	int screenh = duiMainWin.getHeight();
	  
	if (frameLayout.getScale()!=duiMainLayout.getScale()) frameLayout.setScale(duiMainLayout.getScale());
	if ((frameLayout.getLeft()!=screenx) || (frameLayout.getTop()!=screeny) || 
		(frameLayout.getWidth()!=screenw) || (frameLayout.getHeight()!=screenh))
    	frameLayout.resize(screenx, screeny,screenw,screenh);
    					
    // double checks if it allowed the resizing, if not resize back the main window.
    if ((frameLayout.getWidth() != duiMainWin.getWidth()) || (frameLayout.getHeight() != duiMainWin.getHeight())) {
      localResize = FALSE;
      
      frameLayout.onResize(duiMainWin.clientToScreenX(duiMainWin.getLeft()), duiMainWin.clientToScreenY(duiMainWin.getTop()),
      				 frameLayout.getWidth(), frameLayout.getHeight());
    }
  }
  localResize = FALSE;
}

frameLayout.onScale(double newscale) {
	if (newscale == duiMainLayout.getScale()) return;
	if (duiMainLayout) duiMainLayout.setScale(newscale);
}

duiMainWin.onResize(int wx, int wy, int ww, int wh) {
	if (!currwindow_attach_attrib) return;
	if (currwindow_attach_attrib.getData()!="1") return;
	if (concealer.isVisible()) return;
	if (dui_on_transition_attrib.getData()!="0") return; // mainwin group still on transition
	
	syncFrame();
}

duiMainLayout.onMove() {
	mainMove = TRUE;
	
	if (!frameLayout.isVisible()) return;
	if (!currwindow_attach_attrib) return;
	if (currwindow_attach_attrib.getData()!="1") return;
	if (concealer.isVisible()) return;
	if (dui_on_transition_attrib.getData()!="0") return; // mainwin group still on transition
	
	syncFrame();
	mainMove = FALSE;
}

// sync back the window after dui group changes
dui_on_transition_attrib.onDataChanged() {
	if (getData()=="1") return;
	
	syncFrame();
}

duiMainLayout.onSetVisible(boolean on) {
	if (!currwindow_attach_attrib) return;
	if (!frameLayout.isVisible()) return;

	if (on) {
		if (currwindow_tempdetach_attrib.getData()=="1") {
			currwindow_tempdetach_attrib.setData("0");
			delayReattach.start(); // reattach window
		}
		attachButton.setXMLParam("ghost","0");
	} else {
		if (currwindow_attach_attrib.getData()=="1") {
			currwindow_tempdetach_attrib.setData("1");
			currwindow_attach_attrib.setData("0");
		}
		attachButton.setXMLParam("ghost","1");
	}
}

delayReattach.onTimer() {
	stop();
	
	currwindow_attach_attrib.setData("1");
	frameLayout.setFocus();
}

// attrib toggles every 0.1 sec
syncTimer.onTimer() {

	if (!allLoaded) return;
	if (!currwindow_attach_attrib) return;
	if (currwindow_attach_attrib.getData()!="1") return;
	if (concealer.isVisible()) return;
	
	syncFrame();
}

dui_checkfocus_attrib.onDataChanged() {
	if (!allLoaded) return;
	if (!currwindow_attach_attrib) return;
	if (currwindow_attach_attrib.getData()!="1") return;
	if (concealer.isVisible()) return;
	if (!frameLayout.isVisible()) return;
	
	if (frameLayout.isActive()) dui_on_focus_attrib.setData("1");
}

frameLayout.onMove() {
  return;
  if (mainMove) return;
  if (!allLoaded) return;
  if (!currwindow_attach_attrib) return;
  if (currwindow_attach_attrib.getData()!="1") return;
  if (concealer.isVisible()) return;

  syncFrame();
}

setToDUIMode() {
  duiFrame.show();
  wasabiFrame.hide();
  
  frameLayout.setXMLParam("minimum_w","10");
  frameLayout.setXMLParam("minimum_h","10");

  if (content) {
    content.setXmlParam("x", duix);
    content.setXmlParam("y", duiy);
    content.setXmlParam("w", duiw);
    content.setXmlParam("h", duih);
  }
}

resetFromDUIMode() {
  duiMainWin = NULL;
	
  wasabiFrame.show();
  duiFrame.hide();
  
  frameLayout.setXMLParam("minimum_w",min_w);
  frameLayout.setXMLParam("minimum_h",min_h);

  if (content) {
    content.setXmlParam("x", x);
    content.setXmlParam("y", y);
    content.setXmlParam("w", w);
    content.setXmlParam("h", h);
    content.setXmlParam("relatx", rx);
    content.setXmlParam("relaty", ry);
    content.setXmlParam("relatw", rw);
    content.setXmlParam("relath", rh);
  }
}

frameLayout.onGetFocus() {
	if (localFocus) return;
	if (windowName == "") return;
	if (dui_last_focus.getData() == strlower(windowName)) return;
	
	sendDUICommand(windowName, SET_FOCUS, "all");
	dui_last_focus.setData(strlower(windowName));
	frameLayout.setFocus();
}

toggleComponent(string compID) {
	if (compID=="PL") {
		triggerAction(frameLayout,"TOGGLE","guid:pl");
	} else if (compID=="ML") {
		triggerAction(frameLayout,"TOGGLE","guid:ml");
	} else if (compID=="VID") {
		triggerAction(frameLayout,"TOGGLE","guid:{F0816D7B-FFFC-4343-80F2-E8199AA15CC3}");
	} else if (compID=="VIS") {
		triggerAction(frameLayout,"TOGGLE","guid:avs");
	} else if (compID=="EQ") {
		triggerAction(frameLayout,"TOGGLE","eq");
	} else if (compID=="IE") {
		triggerAction(frameLayout,"TOGGLE","ibrowser");
	} else if (compID=="CD") {
		triggerAction(frameLayout,"TOGGLE","cdcover");
	} else if (compID=="CP") {
		triggerAction(frameLayout,"TOGGLE","Config");
	}
}

// **** regular standard-frame scripts **********

System.onSetXuiParam(String param, String value) {
  if (param == "content") {
    setNewGroup(value);
    titlebar = frameGroup.findObject("wasabi.titlebar");
    mouselayer = titlebar.findObject("mousetrap");
  }
  if (param == "padtitleright" || param == "padtitleleft") {
    if (titlebar != NULL) titlebar.setXmlParam(param, value); 
  }
  if (param == "shade") {
    if (mouselayer != NULL) mouselayer.setXmlParam("dblclickaction", "switch;"+value);
    else messagebox("Cannot set shade parameter for StandardFrame object, no mousetrap found", "Skin Error", 0, "");
  }
}

// backward compatibility for prerelease notify trick
frameGroup.onNotify(String cmd, String param, int a, int b) {
  String _command = getToken(cmd, ",", 0);
  String _param = getToken(cmd, ",", 1);
  if (_command == "content" || _command == "padtitleright" || _command == "padtitleleft" || _command == "shade") {
    onSetXuiParam(_command, _param);
  }
}

setNewGroup(String groupid) {
  content = newGroup(groupid);
  if (content == NULL) {
    messagebox("group \"" + groupid + "\" not found", "ButtonGroup", 0, "");
    return;
  }
  
  content.setXmlParam("x", x);
  content.setXmlParam("y", y);
  content.setXmlParam("w", w);
  content.setXmlParam("h", h);
  
  if (currwindow_attach_attrib) {
    if (currwindow_attach_attrib.getData()=="1") {
      content.setXmlParam("x", duix);
      content.setXmlParam("y", duiy);
      content.setXmlParam("w", duiw);
      content.setXmlParam("h", duih);
    }
  }
  content.setXmlParam("relatx", rx);
  content.setXmlParam("relaty", ry);
  content.setXmlParam("relatw", rw);
  content.setXmlParam("relath", rh);
  content.init(frameGroup);
}

Sysmenu.onLeftClick() {
  LayoutStatus _status = frameGroup.findObject("sysmenu.status");
  _status.callme("{system}");
}
