/****************

KameleonDUI - dui-tabs.m

script for the tab feature. (DUI v1.5)


****************/

#include <lib/std.mi>
#include "attribs.m"
#include "utilities.m"
#include "DUI-utilities.m"

function updateTabs();
function int isInsideRect(int mx, int my, int x1, int y1, int x2, int y2);
function string getMoveWindow(int mx, int my);

class button tabButtons;

Global group scriptGroup, tabGroup;

Global layout mainLayout;
Global group duimain, duimain2, duiside, duiside2;
Global int appx1, appy1, appx2, appy2; // coordinates for window regions
Global int mainx1, mainy1, mainx2, mainy2;
Global int main2x1, main2y1, main2x2, main2y2;
Global int sidex1, sidey1, sidex2, sidey2;
Global int side2x1, side2y1, side2x2, side2y2;
Global string onMoveWindowName;

Global text titleText,titleTextTab;
Global guiObject tabMain, tabMainBase;
Global tabButtons tabCurr,tabPL, tabML, tabVID, tabVIS, tabEQ, tabIE, tabCD, tabCP;

Global string currTitle, buttonID;
Global timer onReloadTimer;
Global string nextComp; // next component ID to open after detach/move

Global container tabGhost;
Global layout tabGhostLayout;
Global text tabGhostMessage;
Global int mouseOffsetX, mouseOffsetY;
Global boolean moveWindow;

Global ConfigAttribute currwindow_attach_attrib, currwindow_duiwindow_attrib;

Global timer delayToggle;
Global string toogleID = "";

System.onScriptLoaded() {

	initAttribs();
	
	scriptGroup = getScriptGroup();
	
	tabGroup = scriptGroup.findObject("dui.frame.tabs");
	
	onReloadTimer = new Timer;
	onReloadTimer.setDelay(200);
	
	delayToggle = new Timer;
	delayToggle.setDelay(50);
}

System.onScriptUnloading() {
	// need to clear object variables before unload, this helps prevent
	// script fatal error when used with customobjects.
	
	mainLayout = NULL;
	duimain = NULL;
	duimain2 = NULL;
	duiside = NULL;
	duiside2 = NULL;
	scriptGroup = NULL;
	tabGroup = NULL;
	tabMain = NULL;
	tabMainBase = NULL;
	tabCurr = NULL;
	tabPL = NULL;
	tabML = NULL;
	tabVID = NULL;
	tabVIS = NULL;
	tabEQ = NULL;
	tabIE = NULL;
	tabCD = NULL;
	tabCP = NULL;
	
	if (tabGhost) {
		tabGhost.close(); // kill ghost container
		tabGhost = NULL;
		tabGhostLayout = NULL;
	}
	
	delete onReloadTimer;
	delete delayToggle;
}

scriptGroup.onSetVisible(int on) {
	if (!on) return;
	
	if (!titleText) titleText = scriptGroup.findObject("dui.tab.title.dummy");
	currTitle = titleText.getText();
	currTitle = getToken(currTitle,"\t",0);
	
	if (strleft(currTitle,1)==":") {
		if (!onReloadTimer.isRunning()) onReloadTimer.start();
		return;
	}
	
	if (onReloadTimer.isRunning()) onReloadTimer.stop();
	
	// the rest of onLoad is done here
	
	if (!titleTextTab) titleTextTab = scriptGroup.findObject("dui.tab.titlebar");
	if (!tabMain) tabMain = scriptGroup.findObject("dui.tab.main");
	if (!tabMainBase) tabMainBase = scriptGroup.findObject("dui.tab.main.base");
	if (!tabCurr) tabCurr = scriptGroup.findObject("dui.tab.curr");
	if (!tabPL) tabPL = scriptGroup.findObject("dui.tab.pl");
	if (!tabML) tabML = scriptGroup.findObject("dui.tab.ml");
	if (!tabVID) tabVID = scriptGroup.findObject("dui.tab.vid");
	if (!tabVIS) tabVIS = scriptGroup.findObject("dui.tab.vis");
	if (!tabEQ) tabEQ = scriptGroup.findObject("dui.tab.eq");
	if (!tabIE) tabIE = scriptGroup.findObject("dui.tab.ie");
	if (!tabCD) tabCD = scriptGroup.findObject("dui.tab.cd");
	if (!tabCP) tabCP = scriptGroup.findObject("dui.tab.cp");
	
	if (!currwindow_attach_attrib) currwindow_attach_attrib = custom_page_windows.newAttribute("Attach "+currTitle+" window","1");
	if (!currwindow_duiwindow_attrib)
		currwindow_duiwindow_attrib = custom_page_nonexposed.newAttribute("DUI window "+currTitle,getPrivateString(getSkinName(),"Def DUI:"+windowName,DUI_WIN_MAIN));

	windows_duitab_attrib.onDataChanged();
}

onReloadTimer.onTimer() {
	if (scriptGroup.isVisible()) 
		scriptGroup.onSetVisible(1);
	else
		stop();	
}

scriptGroup.onResize(int x, int y, int w, int h) {
	if (windows_duitab_attrib.getData()!="1") return;
	if (!titleTextTab) titleTextTab = scriptGroup.findObject("dui.tab.titlebar");
	
	if (w <= (titleTextTab.getAutoWidth() + 143))
		tabGroup.hide();
	else
		tabGroup.show();
}

windows_duitab_attrib.onDataChanged() {
	if (getData()=="1") {
		tabGroup.show();
		
		updateTabs();
	} else
		tabGroup.hide();
}

updateTabs() {
	if ((currwindow_duiwindow_attrib==NULL) || (currwindow_duiwindow_attrib==NULL))  {
		onReloadTimer.start(); // grab attribs again
		return;
	}
	
	nextComp = "";
	setPrivateString(getSkinName(),strupper(currTitle)+" Next Window","");
	
	if (windows_duitab_attrib.getData()!="1") return;
	if (currwindow_attach_attrib.getData()!="1") return; // if detached, dont update

	int titleWidth = titleTextTab.getAutoWidth();
	
	titleTextTab.setXMLParam("w", integerToString(titleWidth));
	tabMain.setXMLParam("w", integerToString(titleWidth+30));
	tabMainBase.setXMLParam("w", integerToString(titleWidth+30));
	
	int currX = titleWidth + 21;
	
	ConfigAttribute tabDUIWindowAttrib;
	string currTitleUp1 = getToken(strupper(currTitle)," ",0);
	string currTitleUp2 = getToken(strupper(currTitle)," ",1);
	string currDUIWindow = getToken(currwindow_duiwindow_attrib.getData(),"_",1);
	string otherDUIWindow;
	
	if ((currTitleUp1 != "PLAYLIST") || (currTitleUp2 != "EDITOR") && (windows_pl_attrib.getData()=="1")) {
		tabDUIWindowAttrib = custom_page_nonexposed.newAttribute("DUI window Playlist Editor",DUI_WIN_SIDE);
		if (tabDUIWindowAttrib!=NULL) {
			otherDUIWindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
			if (currDUIWindow == otherDUIWindow) {
				
				tabPL.setXMLParam("x",integerToString(currX));
				tabPL.show();
				currX += 40;
				
				nextComp = "PL";
			} else tabPL.hide();
		} else tabPL.hide();
		tabDUIWindowAttrib = NULL;
	} else tabPL.hide();
	
	if ((currTitleUp1 != "MEDIA") || (currTitleUp2 != "LIBRARY") && (windows_ml_attrib.getData()=="1")) {
		tabDUIWindowAttrib = custom_page_nonexposed.newAttribute("DUI window Media Library",DUI_WIN_MAIN);
		if (tabDUIWindowAttrib!=NULL) {
			otherDUIWindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
			if (currDUIWindow == otherDUIWindow) {
				
				tabML.setXMLParam("x",integerToString(currX));
				tabML.show();
				currX += 40;
				
				if (nextComp=="") nextComp = "ML";
			} else tabML.hide();
		} else tabML.hide();
		tabDUIWindowAttrib = NULL;
	} else tabML.hide();
	
	if ((currTitleUp1 != "VIDEO") && (windows_vid_attrib.getData()=="1")) {
		tabDUIWindowAttrib = custom_page_nonexposed.newAttribute("DUI window Video",DUI_WIN_MAIN);
		if (tabDUIWindowAttrib!=NULL) {
			otherDUIWindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
			if (currDUIWindow == otherDUIWindow) {
				
				tabVID.setXMLParam("x",integerToString(currX));
				tabVID.show();
				currX += 40;
				
				if (nextComp=="") nextComp = "VID";
			} else tabVID.hide();
		} else tabVID.hide();
		tabDUIWindowAttrib = NULL;
	} else tabVID.hide();
	
	if ((currTitleUp1 != "VISUALIZATIONS") && (windows_avs_attrib.getData()=="1")) {
		tabDUIWindowAttrib = custom_page_nonexposed.newAttribute("DUI window Visualizations",DUI_WIN_SIDE2);
		if (tabDUIWindowAttrib!=NULL) {
			otherDUIWindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
			if (currDUIWindow == otherDUIWindow) {
				
				tabVIS.setXMLParam("x",integerToString(currX));
				tabVIS.show();
				currX += 40;
				
				if (nextComp=="") nextComp = "VIS";
			} else tabVIS.hide();
		} else tabVIS.hide();
		tabDUIWindowAttrib = NULL;
	} else tabVIS.hide();
	
	if ((currTitleUp1 != "EQUALIZER") && (windows_eq_attrib.getData()=="1")) {
		tabDUIWindowAttrib = custom_page_nonexposed.newAttribute("DUI window Equalizer",DUI_WIN_MAIN2);
		if (tabDUIWindowAttrib!=NULL) {
			otherDUIWindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
			if (currDUIWindow == otherDUIWindow) {
				
				tabEQ.setXMLParam("x",integerToString(currX));
				tabEQ.show();
				currX += 40;
				
				if (nextComp=="") nextComp = "EQ";

			} else tabEQ.hide();
		} else tabEQ.hide();
		tabDUIWindowAttrib = NULL;
	} else tabEQ.hide();
	
	if ((currTitleUp1 != "INTERNET") || (currTitleUp2 != "BROWSER") && (windows_inet_attrib.getData()=="1")) {
		tabDUIWindowAttrib = custom_page_nonexposed.newAttribute("DUI window Internet Browser",DUI_WIN_MAIN);
		if (tabDUIWindowAttrib!=NULL) {
			otherDUIWindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
			if (currDUIWindow == otherDUIWindow) {
				
				tabIE.setXMLParam("x",integerToString(currX));
				tabIE.show();
				currX += 40;
				
				if (nextComp=="") nextComp = "IE";
			} else tabIE.hide();
		} else tabIE.hide();
		tabDUIWindowAttrib = NULL;
	} else tabIE.hide();
	
	if ((currTitleUp1 != "CD") || (currTitleUp2 != "COVER") && (windows_cdcover_attrib.getData()=="1")) {
		tabDUIWindowAttrib = custom_page_nonexposed.newAttribute("DUI window CD Cover Art",DUI_WIN_SIDE2);
		if (tabDUIWindowAttrib!=NULL) {
			otherDUIWindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
			if (currDUIWindow == otherDUIWindow) {
				
				tabCD.setXMLParam("x",integerToString(currX));
				tabCD.show();
				currX += 40;
				
				if (nextComp=="") nextComp = "CD";
			} else tabCD.hide();
		} else tabCD.hide();
		tabDUIWindowAttrib = NULL;
	} else tabCD.hide();
	
	if ((currTitleUp1 != "CONFIG") || (currTitleUp2 != "PANEL") && (windows_config_attrib.getData()=="1")) {
		tabDUIWindowAttrib = custom_page_nonexposed.newAttribute("DUI window Config Panel",DUI_WIN_MAIN2);
		if (tabDUIWindowAttrib!=NULL) {
			otherDUIWindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
			if (currDUIWindow == otherDUIWindow) {
				
				tabCP.setXMLParam("x",integerToString(currX));
				tabCP.show();
				currX += 40;
				
				if (nextComp=="") nextComp = "CP";
			} else tabCP.hide();
		} else tabCP.hide();
		tabDUIWindowAttrib = NULL;
	} else tabCP.hide();
	
	setPrivateString(getSkinName(),strupper(currTitle)+" Next Window", nextComp);
}

onReceiveDUICommand(string winName, string command, string param) {
	if ((command == OPEN_DUI_GRANTED) || (command == DETACH_WINDOW_GRANTED)) {
		updateTabs();
	}
}

windows_pl_attrib.onDataChanged() { updateTabs(); }
windows_ml_attrib.onDataChanged() { updateTabs(); }
windows_vid_attrib.onDataChanged() { updateTabs(); }
windows_avs_attrib.onDataChanged() { updateTabs(); }
windows_eq_attrib.onDataChanged() { updateTabs(); }
windows_cdcover_attrib.onDataChanged() { updateTabs(); }
windows_inet_attrib.onDataChanged() { updateTabs(); }
windows_config_attrib.onDataChanged() { updateTabs(); }

// *** toggle, drag and drop scripts ***
int isInsideRect(int mx, int my, int x1, int y1, int x2, int y2) {
	return (mx >= x1) && (mx <= x2) && (my >=y1) && (my <=y2); 
}

string getMoveWindow(int mx, int my) {
	
	if (!isInsideRect(mx, my, appx1, appy1, appx2, appy2)) return "DETACH";
	if (isInsideRect(mx, my, mainx1, mainy1, mainx2, mainy2)) return DUI_WIN_MAIN;
	if (isInsideRect(mx, my, main2x1, main2y1, main2x2, main2y2)) return DUI_WIN_MAIN2;
	if (isInsideRect(mx, my, sidex1, sidey1, sidex2, sidey2)) return DUI_WIN_SIDE;
	if (isInsideRect(mx, my, side2x1, side2y1, side2x2, side2y2)) return DUI_WIN_SIDE2;
	
	return "";
}


tabButtons.onLeftButtonDown(int x, int y) {
	buttonID = strupper(getToken(getID(),".",2));
	
	mouseOffsetX = x-getLeft();
	mouseOffsetY = y-getTop();
	
	moveWindow = FALSE;
	
	if (!mainLayout) {
		mainLayout = getContainer("main").getLayout("normal");
		duimain = mainLayout.findObject("dui.main.content");
		duimain2 = mainLayout.findObject("dui.main2.content");
		duiside = mainLayout.findObject("dui.side.content");
		duiside2 = mainLayout.findObject("dui.side2.content");
		
		
	}
	
	group duimainparent = duimain.getParent();
	group duisideparent = duiside.getParent();
	
	float mainScale = mainLayout.getScale();
	
	appx1 = mainLayout.getLeft();
	appy1 = mainLayout.getTop();
	appx2 = appx1 + mainLayout.getWidth()*mainScale;
	appy2 = appy1 + mainLayout.getHeight()*mainScale;
	
	mainx1 = 0; mainy1 = 0; mainx2 = 0; mainy2 = 0;
	main2x1 = 0; main2y1 = 0; main2x2 = 0; main2y2 = 0;
	sidex1 = 0; sidey1 = 0; sidex2 = 0; sidey2 = 0;
	side2x1 = 0; side2y1 = 0; side2x2 = 0; side2y2 = 0;
	
	if (duimain.isVisible()) {
		mainx1 = duimain.getLeft()*mainScale + mainLayout.getLeft();
		mainy1 = duimain.getTop()*mainScale + mainLayout.getTop();
		mainx2 = mainx1 + duimain.getWidth()*mainScale;
		mainy2 = mainy1 + duimain.getHeight()*mainScale;
	} 
	if (duimain2.isVisible()) {
		main2x1 = duimain2.getLeft()*mainScale + mainLayout.getLeft();
		main2y1 = duimain2.getTop()*mainScale + mainLayout.getTop();
		main2x2 = main2x1 + duimain2.getWidth()*mainScale;
		main2y2 = main2y1 + duimain2.getHeight()*mainScale;
	}
	if (duiside.isVisible()) {
		sidex1 = duiside.getLeft()*mainScale + mainLayout.getLeft();
		sidey1 = duiside.getTop()*mainScale + mainLayout.getTop();
		sidex2 = sidex1 + duiside.getWidth()*mainScale;
		sidey2 = sidey1 + duiside.getHeight()*mainScale;
	}
	if (duiside2.isVisible()) {
		side2x1 = duiside2.getLeft()*mainScale + mainLayout.getLeft();
		side2y1 = duiside2.getTop()*mainScale + mainLayout.getTop();
		side2x2 = side2x1 + duiside2.getWidth()*mainScale;
		side2y2 = side2y1 + duiside2.getHeight()*mainScale;
	} 

	if (!duimain.isVisible()) {
		if (duimain2.isVisible()) {
			mainx1 = main2x1;
			mainy1 = main2y1;
			mainx2 = main2x2;
			mainy2 = mainy1 + 20;
			
			main2y1 += 20;
		} else {
			mainx1 = duisideparent.getLeft()*mainScale + mainLayout.getLeft();
			mainy1 = duisideparent.getTop()*mainScale + mainLayout.getTop();
			mainx2 = mainx1 + 20;
			mainy2 = mainy1 + duisideparent.getHeight()*mainScale*0.7;
			
			main2x1 = mainx1;
			main2y1 = mainy2 + 1;
			main2x2 = mainx2;
			main2y2 = mainy1 + duisideparent.getHeight()*mainScale;
			
			sidex1 += 20;
			side2x2 += 20;
		}
	} else if (!duimain2.isVisible()) {
		main2x1 = mainx1;
		main2y1 = mainy2-20;
		main2x2 = mainx2;
		main2y2 = mainy2;
		
		mainy2 -= 20;
	}
	if (!duiside.isVisible()) {
		if (duiside2.isVisible()) {
			sidex1 = side2x1;
			sidey1 = side2y2 - 20;
			sidex2 = side2x2;
			sidey2 = side2y2;
			
			side2y2 -= 20;
		} else {
			side2x2 = duimainparent.getLeft()*mainScale + mainLayout.getLeft() + duimainparent.getWidth()*mainScale;
			side2y1 = duimainparent.getTop()*mainScale + mainLayout.getTop();
			side2x1 = side2x2 - 20;
			side2y2 = side2y1 + duimainparent.getHeight()*mainScale*0.3;
			
			sidex1 = side2x1;
			sidey1 = side2y2 + 1;
			sidex2 = side2x2;
			sidey2 = side2y1 + duimainparent.getHeight()*mainScale;
			
			mainx2 -= 20;
			main2x2 -=20;
		}
	} else if (!duiside2.isVisible()) {
		side2x1 = sidex1;
		side2y1 = sidey1;
		side2x2 = sidex2;
		side2y2 = sidey1 + 20;
		
		sidey1 += 20;
	}
}

tabButtons.onMouseMove(int x, int y) {
	string currID = strupper(getToken(getID(),".",2));
	text ghosttitle;
	
	if (currID!=buttonID) return;
	
	if (!tabGhost) {
		if (buttonID=="CURR") {
			onMoveWindowName = currTitle;
		} else if (buttonID=="PL") {
			onMoveWindowName = "Playlist";
		} else if (buttonID=="ML") {
			onMoveWindowName = "Media Library";
		} else if (buttonID=="VID") {
			onMoveWindowName = "Video";
		} else if (buttonID=="VIS") {
			onMoveWindowName = "Visualizations";
		} else if (buttonID=="EQ") {
			onMoveWindowName = "Equalizer";
		} else if (buttonID=="IE") {
			onMoveWindowName = "Internet Browser";
		} else if (buttonID=="CD") {
			onMoveWindowName = "CD Cover Art";
		} else if (buttonID=="CP") {
			onMoveWindowName = "Config Panel";
		}
		
		int diffx = x-mouseOffsetX-getLeft();
		int diffy = y-mouseOffsetY-getTop();
		
		if ((diffx > -10) && (diffx < 10)) return;
		if ((diffy > -10) && (diffy < 10)) return;
		
		moveWindow = TRUE;
			
		tabGhost = newDynamicContainer("dui.tab.ghost");
		
		if (tabGhost) {
			tabGhostLayout = tabGhost.getLayout("normal");
			
			if (tabGhostLayout) {
				ghosttitle = tabGhostLayout.getObject("tab.ghost.titlebar");
				tabGhostMessage = tabGhostLayout.getObject("tab.ghost.message");
				
	
				ghosttitle.setText(onMoveWindowName);
			}
		}
	}
	
	if (tabGhostLayout) {
		if (!tabGhostLayout.isVisible()) tabGhostLayout.show();
		int mx = getMousePosX();
		int my = getMousePosY();
		
		tabGhostLayout.resize(mx - mouseOffsetX, my - mouseOffsetY, ghostTitle.getAutoWidth()+40,50);
		
		string ret = getMoveWindow(mx, my);
		
		if (ret == "DETACH") tabGhostMessage.setText("Detach Window");
		else if (ret == DUI_WIN_MAIN) tabGhostMessage.setText("DUI Main");
		else if (ret == DUI_WIN_MAIN2) tabGhostMessage.setText("DUI Bottom Main");
		else if (ret == DUI_WIN_SIDE) tabGhostMessage.setText("DUI Side");
		else if (ret == DUI_WIN_SIDE2) tabGhostMessage.setText("DUI Top Side");
		
		if (ret == "") {
			tabGhostMessage.setText("Move to...");
			tabGhostLayout.setAlpha(128);
		} else 
			tabGhostLayout.setAlpha(200);
	}
}

tabButtons.onLeftButtonUp(int x, int y) {
	string action = getMoveWindow(getMousePosX(), getMousePosY());

	if (buttonID!="CURR") // if not current, dont toggle next component
		setPrivateString(getSkinName(),strupper(currTitle)+" Next Window","");
	
	if (onMoveWindowName=="Playlist") onMoveWindowName="Playlist Editor";
	
	if (moveWindow) {
		if (action == "DETACH") {
			setPrivateInt(getSkinName(),strupper(onMoveWindowName)+"Last X", getMousePosX()-10); // place detached window where the mouse is
			setPrivateInt(getSkinName(),strupper(onMoveWindowName)+"Last Y", getMousePosY()-10); 
			
			configAttribute attachAttrib = custom_page_windows.getAttribute("Attach " + onMoveWindowName + " window");
			if (attachAttrib) attachAttrib.setData("0");
		} else {
			configAttribute tabDUIWindowAttrib;
			string currduiwindow, otherduiwindow;
			string actionduiwindow = getToken(action,"_",1);
			
			if (buttonID=="CURR") {
				currduiwindow = getToken(currwindow_duiwindow_attrib.getData(),"_",1);
				if (actionduiwindow != currduiwindow) {
					if ((action==DUI_WIN_MAIN) || (action==DUI_WIN_MAIN2) || (action==DUI_WIN_SIDE) || (action==DUI_WIN_SIDE2))
						currwindow_duiwindow_attrib.setData(action);
				}
			} else {
				tabDUIWindowAttrib = custom_page_nonexposed.getAttribute("DUI window "+onMoveWindowName);
				if (tabDUIWindowAttrib!=NULL) {
					otherduiwindow = getToken(tabDUIWindowAttrib.getData(),"_",1);
					if (actionduiwindow != otherduiwindow) {
						if ((action==DUI_WIN_MAIN) || (action==DUI_WIN_MAIN2) || (action==DUI_WIN_SIDE) || (action==DUI_WIN_SIDE2))
							tabDUIWindowAttrib.setData(action);
					}
				}
				tabDUIWindowAttrib = NULL;
			}
		}
	}
		
	toogleID = buttonID;
	buttonID = "";
	delayToggle.onTimer();
	
	if (tabGhost) {
		tabGhost.close(); // kill ghost container
		tabGhost = NULL;
		tabGhostLayout = NULL;
	}
	
	updateTabs();
}

delayToggle.onTimer() {
	if (isRunning()) stop();
	
	if (toogleID=="PL") {
		triggerAction(scriptGroup,"TOGGLE","guid:pl");
	} else if (toogleID=="ML") {
		triggerAction(scriptGroup,"TOGGLE","guid:ml");
	} else if (toogleID=="VID") {
		triggerAction(scriptGroup,"TOGGLE","guid:{F0816D7B-FFFC-4343-80F2-E8199AA15CC3}");
	} else if (toogleID=="VIS") {
		triggerAction(scriptGroup,"TOGGLE","guid:avs");
	} else if (toogleID=="EQ") {
		triggerAction(scriptGroup,"TOGGLE","eq");
	} else if (toogleID=="IE") {
		triggerAction(scriptGroup,"TOGGLE","ibrowser");
	} else if (toogleID=="CD") {
		triggerAction(scriptGroup,"TOGGLE","cdcover");
	} else if (toogleID=="CP") {
		triggerAction(scriptGroup,"TOGGLE","Config");
	}
}
