/*************************************************************

  KameleonDUI - beatvis.m
  by Leechbite

  script for beat visualization animation.
  

*************************************************************/

#include <lib/std.mi>
#include "attribs.m"

function updateVis();

Global animatedLayer leftBeat = NULL, rightBeat = NULL;
Global boolean monoMode = 0;
Global int lastLeftVU, lastRightVU, maxDev;
Global int maxVU;

Global timer timerRefresh;
Global group scriptGroup;

Global int leftNumFrames, rightNumFrames;

System.onScriptLoaded() {
	initAttribs();
	
	scriptGroup = getScriptGroup();
	string param = getParam();

	leftBeat = scriptGroup.getObject(getToken(param,",",0));
	rightBeat = scriptGroup.getObject(getToken(param,",",1));
	monoMode = (rightBeat == NULL);

	if (rightBeat) rightNumFrames = rightBeat.getLength()-1;
	if (leftBeat) leftNumFrames = rightBeat.getLength()-1;
	
	maxDev = stringToInteger(getToken(param,",",2));
	if (maxDev < 20) maxDev = 20;
	maxVU = 100;
	
	timerRefresh = new Timer;
	timerRefresh.setDelay(50);
	
	beat_off_attrib.onDataChanged();
}

System.onScriptUnloading() {
	timerRefresh.stop();
	delete timerRefresh;
	
	leftBeat = NULL;
	rightBeat = NULL;
	scriptGroup = NULL;
}

beat_off_attrib.onDataChanged() {
	if (getData()=="0") {
		if (leftBeat) { leftBeat.gotoFrame(0); leftBeat.setAlpha(255); }
		if (rightBeat) { rightBeat.gotoFrame(0); rightBeat.setAlpha(255); }
		timerRefresh.start();
	} else {
		timerRefresh.stop();
		if (leftBeat) { leftBeat.gotoFrame(leftNumFrames); leftBeat.setAlpha(64); }
		if (rightBeat) { rightBeat.gotoFrame(rightNumFrames); rightBeat.setAlpha(64); }
	}
}

scriptGroup.onSetVisible(int on) {
	if (on && beat_off_attrib.getData()=="0") 
		timerRefresh.start(); 
	else
		timerRefresh.stop();
}

updateVis() {
	int leftVU, rightVU, x;
	
	if (beat_srcVU_attrib.getData() == "1") {
		
		leftVU = getLeftVUMeter();
		rightVU = getRightVUMeter();
		
	} else if (beat_srcBass_attrib.getData() == "1") {
		
		leftVU = 0;
		rightVU = 0;
		
		for (x=0; x<=4; x++)
		{
			leftVU = leftVU+getVisBand(0,x);
			rightVU = rightVU+getVisBand(0,x);
		}
		leftVU = leftVU/5;
		rightVU = rightVU/5;
	} else {
		
		leftVU = 0;
		rightVU = 0;
		
		for (x=0; x<=5; x++)
		{
			leftVU = leftVU+getVisBand(0,75-x*2);
			rightVU = rightVU+getVisBand(0,75-x*2);
		}
		leftVU = leftVU/5;
		rightVU = rightVU/5;
	}
	
	if (monoMode) leftVU = (leftVU + rightVU)/2;
	
	if ((leftVU - lastLeftVU) > maxDev) leftVU = lastLeftVU + maxDev;
	if ((leftVU - lastLeftVU) < -maxDev) leftVU = lastLeftVU - maxDev;
	
	if ((rightVU - lastRightVU) > maxDev) rightVU = lastRightVU + maxDev;
	if ((rightVU - lastRightVU) < -maxDev) rightVU = lastRightVU - maxDev;
	
	if (leftVU < 0) leftVU = 0;
	if (leftVU > 255) leftVU = 255;
	if (rightVU < 0) rightVU = 0;
	if (rightVU > 255) rightVU = 255;

	lastLeftVU = leftVU;
	lastRightVU = rightVU;
	
	if (leftVU > maxVU) maxVU = leftVU;
	if (rightVU > maxVU) maxVU = rightVU;
	leftVU = leftVU*255/maxVU;
	rightVU = rightVU*255/maxVU;

	if (leftBeat) leftBeat.gotoFrame(leftNumFrames * leftVU / 255);
	if (rightBeat) rightBeat.gotoFrame(rightNumFrames * rightVU / 255);
}

system.onTitleChange(string newt) {
	maxVU = 100;
}

timerRefresh.onTimer() {
	updateVis();
}