/*************************************************************

  KameleonDUI - incrementaldim.m
  by Leechbite

  all-purpose script for autoresizing guiObjects into incremental
  width and/or height depending on dimension of parent group.
  incremental width of 10px will autoresize the object
  by 10px (0, 10, 20, 30 etc.). 
  script also does alignment of object to parentgroup.
  
  takes 11 parameters: (eq. param="objID,incW,incH,minW,minH,maxW,maxH,align,valign,addbuttonspacex,addbuttonspacey")
  
  objID = id of object to show and hide.
  incW = incremental width, set to 1 to disable.
  incH = incremental heihgt, set to 1 to disable.
  minW,minH,maxW,maxH = minimum and maximum width and height.
  halign = horizonal alignment (LEFT, CENTER, RIGHT) default: LEFT
  valign = vertical alignment (TOP, MIDDLE, BOTTOM) default: TOP
  addbuttonspacex, addbuttonspacey = if the current width/height is less the maximum, a space is
  		provided for a button with a size of addbuttonspacex on the x-axis and addbuttonspacey on the y-axis
  
*************************************************************/

#include <lib/std.mi>

Global group parentGroup;
Global guiObject obj;
Global int incW, incH;
Global int minW, minH, maxW, maxH;
Global int alignH, alignV; // integer: 0 - left/top; 1 - center/middle; 2 - right/bottom
Global int addbuttonspacex,addbuttonspacey;

System.onScriptLoaded() {
	parentGroup = getScriptGroup();
	string param = getParam();

	obj = parentGroup.findObject(getToken(param,",",0));
	incW = stringToInteger(getToken(param,",",1));
	incH = stringToInteger(getToken(param,",",2));
	if (incW <= 0) incW = 1; // if not given, set to 1
	if (incH <= 0) incH = 1;
	
	minW = stringToInteger(getToken(param,",",3));
	minH = stringToInteger(getToken(param,",",4));
	maxW = stringToInteger(getToken(param,",",5));
	maxH = stringToInteger(getToken(param,",",6));
	
	string t = strupper(getToken(param,",",7));
	if (t=="CENTER") 
		alignH = 1;
	else if (t=="RIGHT")
		alignH = 2;
	else
		alignH = 0;
		
	t = strupper(getToken(param,",",8));
	if (t=="MIDDLE") 
		alignV = 1;
	else if (t=="BOTTOM")
		alignV = 2;
	else
		alignV = 0;
	
		
	addbuttonspacex = stringToInteger(getToken(param,",",9));
	addbuttonspacey = stringToInteger(getToken(param,",",10));
	parentGroup.onResize(parentGroup.getLeft(),parentGroup.getTop(),
						parentGroup.getWidth(),parentGroup.getHeight());
						
}

System.onScriptUnloading() {
	return;
}

parentGroup.onResize(int x, int y, int w, int h) {
	if (w < maxW) w = w - addbuttonspacex;
	if (h < maxH) h = h - addbuttonspacey;
	int neww = w / incW;
	neww = neww * incW;
	
	int newh = h / incH;
	newh = newh * incH;

	if (neww < minW) newW = minW;
	if (newh < minH) newH = minH;
	if (neww > maxW) newW = maxW;
	if (newh > maxH) newH = maxH;
	
	obj.setXMLParam("w", integerToString(neww));
	obj.setXMLParam("h", integerToString(newh));
	
	if (alignH == 0) // left alignment
		obj.setXMLParam("x", "0");
	else if (alignH == 1) // center alignment
		obj.setXMLParam("x", integerToString((w-neww)/2));
	else 			// right alignment
		obj.setXMLParam("x", integerToString(w-neww));
		
	if (alignV == 0) //  top alignment
		obj.setXMLParam("y", "0");
	else if (alignV == 1) // middle alignment
		obj.setXMLParam("y", integerToString((h-newh)/2));
	else 			// bottom alignment
		obj.setXMLParam("y", integerToString(h-newh));
}