#include <lib/std.mi>
#include "attribs.m"
#include "utilities.m"

Function reset();
Function createNotifier();
Function showNotifier(Int w);
Function onNext();

Function Int fillNextTrackInfo(String corneroverride);
Function Int fillCustomInfo(String customstring);

Function checkPref(int bypassfs);

Global Container notifier_container;
Global Layout notifier_layout;
Global Group notifiercontent;
Global Timer notifier_timer, songtimer;
Global String last_autotitle;

Global boolean onClose = false, endOfTrackShow = false, counter = 0;
Global int currPos = 0; // 00 - bottom right, 01 - bottom left, 10 - top right, 11 - top left

// ------------------------------------------------------------------------------
// init
// ------------------------------------------------------------------------------
System.onScriptLoaded() {
  initAttribs();
  notifier_timer = new Timer;

  songtimer = new Timer;
  songtimer.setDelay(1000);
  notifier_showend_attrib.onDataChanged();
}

// ------------------------------------------------------------------------------
// shutdown
// ------------------------------------------------------------------------------
System.onScriptUnloading() {
  delete notifier_timer;
  delete songtimer;
}

// ------------------------------------------------------------------------------
// called by the system when the global hotkey for notification is pressed
// ------------------------------------------------------------------------------
System.onShowNotification() {
  if (checkPref(1)) return;
  createNotifier();
  String str;
  if (getStatus() == STATUS_PLAYING) str = "Playing";
  if (getStatus() == STATUS_PAUSED) str = "Playback Paused";
  if (getStatus() == STATUS_STOPPED) str = "Playback Stopped";
  showNotifier(fillNextTrackInfo(str));
  complete; // prevents other scripts from getting the message
  return 1; // tells anybody else that might watch the returned value that, yes, we implemented that
}

// ------------------------------------------------------------------------------
// called by the system when the title for the playing item changes, this could be the result of the player
// going to the next track, or of an update in the track title
// ------------------------------------------------------------------------------
System.onTitleChange(String newtitle) {
  if (last_autotitle == newtitle) return;
  if (StrLeft(newtitle, 1) == "[") {
    if (StrLeft(newtitle, 7) == "[Buffer" ||
        StrLeft(newtitle, 4) == "[ICY") return;
  }
  last_autotitle = newtitle;
  onNext();
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the next button
// ------------------------------------------------------------------------------
onNext() {
  endOfTrackShow = false;
  if (checkPref(0)) return;
  createNotifier();
  showNotifier(fillNextTrackInfo(""));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the play button
// ------------------------------------------------------------------------------
System.onPlay() {
  endOfTrackShow = false;
  if (checkPref(0)) return;
  createNotifier();
  showNotifier(fillNextTrackInfo("Playing"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the pause button
// ------------------------------------------------------------------------------
System.onPause() {
  if (checkPref(0)) return;
  createNotifier();
  showNotifier(fillCustomInfo("Playback Paused"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the pause button again
// ------------------------------------------------------------------------------
System.onResume() {
  if (checkPref(0)) return;
  createNotifier();
  showNotifier(fillNextTrackInfo("Resuming Playback"));
}

// ------------------------------------------------------------------------------
// called by the system when the user clicks the play button
// ------------------------------------------------------------------------------
System.onStop() {
  endOfTrackShow = false;
  if (checkPref(0)) return;
  createNotifier();
  showNotifier(fillCustomInfo("End of Playback"));
}

// ------------------------------------------------------------------------------
//  ******** used for display at end of track. ;) my own addition.
// ------------------------------------------------------------------------------

songtimer.onTimer() {
  if (endOfTrackShow) return;
  if (getStatus() == 0) return;

  int songlngth = getPlayItemLength();
  int notifierlength =  4000 + stringToInteger(attrXfadeTime.getData())*1000 + 
			stringToInteger(notifier_holdtime_attrib.getData());

  int dif = getPlayItemLength() - getPosition() - notifierlength;
  if ((dif <= 8000) && (dif >= 0)) {
	endOfTrackShow = true;
	if (checkPref(0)) return;
	createNotifier();
	showNotifier(fillNextTrackInfo("Playing"));
  }
}

// ------------------------------------------------------------------------------
// checks if we should display anything
// ------------------------------------------------------------------------------
Int checkPref(int bypassfs) {
  if (!bypassfs && notifier_disablefullscreen_attrib.getData() == "1" && isVideoFullscreen()) return 1;
  if (notifier_never_attrib.getData() == "1") return 1;
  if (notifier_minimized_attrib.getData() == "1" && !isMinimized()) return 1;
  if (notifier_windowshade_attrib.getData() == "1") {
    if (isMinimized()) return 0;
    Container c = getContainer("main");
    if (!c) return 1;
    Layout l = c.getCurLayout();
    if (!l) return 1;
    if (l.getId() != "shade") return 1;
  }
  return 0;
}

// ------------------------------------------------------------------------------
// fade in/out completed
// ------------------------------------------------------------------------------
notifier_layout.onTargetReached() {
  int a = notifier_layout.getAlpha();
  if (a == 255) {
    notifier_timer.setDelay(StringToInteger(notifier_holdtime_attrib.getData()));
    notifier_timer.start();
  }
  else if (a == 0) {
    reset();
  }
}

// ------------------------------------------------------------------------------
// hold time elapsed
// ------------------------------------------------------------------------------
notifier_timer.onTimer() {
  stop();
  onClose = true;
  if (notifier_layout.isTransparencySafe() && notifier_anim_fade_attrib.getData()=="1") {
    notifier_layout.setTargetA(0);
    notifier_layout.setTargetSpeed(1.0);
    notifier_layout.gotoTarget();
  } else if (notifier_anim_slidevert_attrib.getData()=="1") {
    if (currpos==10 || currpos==11)  // see variable declaration for meaning
	  notifiercontent.setTargetY(-100);
	else
	  notifiercontent.setTargetY(100);
	notifiercontent.setTargetX(10);
	notifiercontent.setTargetW(-20);
	notifiercontent.setTargetSpeed(2.0);
	notifiercontent.gotoTarget();    
  } else if (notifier_anim_slidehort_attrib.getData()=="1") {
	int nwidth = notifier_Layout.getWidth();
	  
    notifiercontent.setXMLParam("relatw","0");
    notifiercontent.setXMLParam("w",integerToString(nwidth-20));

    if (currpos==11 || currpos==1) // see variable declaration for meaning
	  notifiercontent.setTargetX(-nwidth);
	else
	  notifiercontent.setTargetX(nwidth);
	notifiercontent.setTargetY(10);
	notifiercontent.setTargetW(nwidth-20);
	notifiercontent.setTargetSpeed(2.0);
	notifiercontent.gotoTarget();    
  } else {
    reset();
  }
}

notifiercontent.onTargetReached() {
	if (onClose) 
		reset();
	else {
		notifier_timer.setDelay(StringToInteger(notifier_holdtime_attrib.getData()));
    	notifier_timer.start();
	}
}

// ------------------------------------------------------------------------------
// when notifier is clicked, bring back the app from minimized state if its minimized and focus it
// ------------------------------------------------------------------------------
notifier_layout.onLeftButtonDown(int x, int y) {
  notifier_timer.stop();
  notifier_layout.cancelTarget();
  reset();
  restoreApplication();
  activateApplication();
}

// ------------------------------------------------------------------------------
// close the notifier window, destroys the container automatically because it's dynamic
// ------------------------------------------------------------------------------
// this notifier is not closed to prevent some other scripts from unloading (cd-cover, kameleon-background)
// unloading a script in a middle of another script (like the popUpMenu) causes Assertion Faults
reset() {
  notifier_timer.stop();
  notifier_layout.hide();
}

// ------------------------------------------------------------------------------
createNotifier() {
  if (notifier_container == NULL) {
    notifier_container = newDynamicContainer("notifier");
    if (!notifier_container) return; // reinstall duh!

    notifier_layout = notifier_container.getLayout("normal");

    if (!notifier_layout) return; // reinstall twice, man
  } else {
    notifier_layout.cancelTarget();
    notifier_timer.stop();
  }
  
}

// ------------------------------------------------------------------------------
showNotifier(int w) {
  w = w + 74;
  int layx,layy;
  onClose = false;
  if (!notifier_layout) return;

  group textgroup=NULL;
  group cdcover = notifier_layout.findObject("notifier.cdcover");
  textgroup = notifier_layout.findObject("notifier.text");
  notifiercontent = NULL;
  notifiercontent = notifier_layout.getObject("notifier.content");

  if (notifier_cdcover_attrib.getData()=="1") {
    w = w + 65;
    if (cdcover) {cdcover.show(); }
    if (textgroup) {
      textgroup.setXMLParam("x", "92");
      textgroup.setXMLParam("w", "-119");
    }
  } else {
    if (cdcover) cdcover.hide();
    if (textgroup) {
      textgroup.setXMLParam("x", "27");
      textgroup.setXMLParam("w", "-54");
    }
  }

  if ((notifier_location_topleft_attrib.getData()=="1") || (notifier_location_topright_attrib.getData()=="1")) {
    layy = getViewportTop();
    currPos = 10;		// see variable declaration for meaning
  } else {
    layy = getViewportHeight() + getViewportTop() - 100 ;
    currPos = 00;
  }

  if ((notifier_location_topleft_attrib.getData()=="1") || (notifier_location_bottomleft_attrib.getData()=="1")) {
    layx = getViewportLeft();
    currPos = currPos + 1;
  } else
    layx = getViewportWidth() + getViewportLeft() - w;


  if ((notifier_showend_attrib.getData()=="1") && (!songtimer.isRunning())) songtimer.start();

  boolean firstOpen = !notifier_layout.isVisible(); // checks if notifier is already visible
    
  if (firstOpen || notifier_noanim_attrib.getData()=="1") notifier_layout.resize(layx, layy, w, 100);
  notifier_timer.stop();
  
  if (firstOpen) {
	notifiercontent.setXMLParam("x","10"); // restore original notifiercontent params
	notifiercontent.setXMLParam("y","10");
	notifiercontent.setXMLParam("w","-20");
	notifiercontent.setXMLParam("h","80");
	notifiercontent.setXMLParam("relatx","0");
	notifiercontent.setXMLParam("relaty","0");
	notifiercontent.setXMLParam("relatw","1");
	notifiercontent.setXMLParam("relath","0");
	notifiercontent.setTargetX(10);
	notifiercontent.setTargetY(10);
	//notifiercontent.setTargetW(notifiercontent.getWidth());
	  
	if (notifier_anim_slidevert_attrib.getData()=="1") {

	  if ((notifier_location_topleft_attrib.getData()=="1") || (notifier_location_topright_attrib.getData()=="1")) 
	    notifiercontent.setXMLParam("y","-100");
	  else
	    notifiercontent.setXMLParam("y","100");

 	  notifier_layout.setAlpha(255);
      notifier_layout.show();
      notifiercontent.show();
	    
	  notifiercontent.cancelTarget();
	  notifiercontent.setXMLParam("relaty","0");
	  notifiercontent.setTargetX(10);
	  notifiercontent.setTargetY(10);
	  notifiercontent.setTargetW(-20);
	  notifiercontent.setTargetSpeed(2.0);
	  notifiercontent.gotoTarget();
	  
    } else if (notifier_anim_slidehort_attrib.getData()=="1") {
	  notifiercontent.setXMLParam("relatw","0");
	  notifiercontent.setXMLParam("w",integerToString(w-20));
	  if ((notifier_location_topleft_attrib.getData()=="1") || (notifier_location_bottomleft_attrib.getData()=="1"))
	    notifiercontent.setXMLParam("x",integerToString(-w));
	  else
	    notifiercontent.setXMLParam("x",integerToString(w));
	  notifiercontent.setTargetX(10);
	  notifiercontent.setTargetY(10);
	  notifiercontent.setTargetW(w-20);
	  notifiercontent.setTargetSpeed(2.0);

	  notifier_layout.setAlpha(255);
      notifier_layout.show();

	  notifiercontent.show();
	  notifiercontent.gotoTarget();
	  
    }
  }
  if (notifier_layout.isTransparencySafe() && notifier_anim_fade_attrib.getData()=="1") {
	notifiercontent.setXMLParam("relatw","1");
	notifiercontent.setXMLParam("w","-20");

    notifier_layout.show();
    if (firstOpen) notifier_layout.setAlpha(0);
    notifier_layout.setTargetA(255);
    notifier_layout.setTargetX(layx);
    notifier_layout.setTargetY(layy);
    notifier_layout.setTargetW(w);
    notifier_layout.setTargetH(100);
    notifier_layout.setTargetSpeed(1.0);
    notifier_layout.gotoTarget();
  } else { // if not first open and not fade animation, resize the current shown notifier. ;)
    notifier_layout.setAlpha(255);
    notifier_layout.show();
    if (!firstOpen && notifier_noanim_attrib.getData()!="1") {
	  if (notifiercontent.isGoingToTarget()) {
		if (notifier_anim_slidehort_attrib.getData()=="1") {
			notifiercontent.setXMLParam("relatw","0");
			notifiercontent.setXMLParam("w",integerToString(notifiercontent.getWidth()));
  	  		notifiercontent.setTargetW(w-20);
		} else {
	      notifiercontent.setXMLParam("relatw","1");
  		  notifiercontent.setXMLParam("w","-20");
		}
		  
	    notifiercontent.setTargetX(10);
	    notifiercontent.setTargetY(10);
	    notifiercontent.setTargetSpeed(1.0);
	    notifiercontent.gotoTarget();
      } else { 
        notifiercontent.cancelTarget();
	    notifiercontent.setXMLParam("x","10");
        notifiercontent.setXMLParam("y","10");
        notifiercontent.setXMLParam("relatw","1");
        notifiercontent.setXMLParam("w","-20");
      }

      notifier_layout.setTargetA(255);
      notifier_layout.setTargetX(layx);
      notifier_layout.setTargetY(layy);
      notifier_layout.setTargetW(w);
      notifier_layout.setTargetH(100);
      notifier_layout.setTargetSpeed(1.0);
      notifier_layout.gotoTarget();
    }
    if (notifier_noanim_attrib.getData()=="1") {
		notifier_timer.setDelay(StringToInteger(notifier_holdtime_attrib.getData()));
		notifier_timer.start();
    }
  }
  

}

notifier_showend_attrib.onDataChanged() {
  if (getData()=="1")
    songtimer.start();
  else
    songtimer.stop();
}

// ------------------------------------------------------------------------------
Int fillNextTrackInfo(String corneroverride) {
  if (!notifier_layout) return;
	
  Int maxv = 0;
  Int stream = 0;
  Group p = notifier_layout;
  Text plentry = p.findObject("plentry");
  Text nexttrack = p.findObject("nexttrack");
  Text _title = p.findObject("title");
  Text album = p.findObject("album");
  Text year = p.findObject("year"); if (year) year.show();
  Text artist = p.findObject("artist");
  Text endofplayback = p.findObject("endofplayback");

  DebugString("got callback for " + getPlayItemString(), 0);
  if (StrLeft(getPlayItemString(), 7) == "http://") stream = 1;

  if (endofplayback) endofplayback.hide();

  if (plentry) { plentry.setText(integerToString(getPlaylistIndex()+1)+"/"+integerToString(getPlaylistLength())); plentry.show(); }
  if (nexttrack) {
    if (corneroverride == "") {
      if (!stream) {
        if (!isVideo())
          nexttrack.setText("New track");
        else
          nexttrack.setText("New video");
      }
      else nexttrack.setText("On air");
    } else nexttrack.setText(corneroverride);
    nexttrack.show();
  }
  if (_title) {
    String str;
    if (!stream) {
      _title.setXmlParam("ticker", "0");
      _title.setXmlParam("display", "");
      str = getPlayitemMetaDataString("title"); 
      if (str == "") str = getPlayitemDisplayTitle();
      String l = getPlayItemMetaDataString("length");
      if (l != "") {
        str += " (" + integerToTime(stringtointeger(l)) + ")";
      }
      _title.setText(str);
    } else {
      _title.setXmlParam("ticker", "1");
      _title.setXmlParam("display", "songtitle");
      _title.setText("");
    }
    _title.show(); 
  }
  if (artist) { 
    if (!stream) {
      if (isVideo())
        artist.setText(""); 
      else
        artist.setText(getPlayitemMetaDataString("artist")); 
    } else {
      if (isVideo())
        artist.setText("Internet TV"); 
      else
        artist.setText("Internet Radio"); 
    }
    artist.show(); 
  }
  if (album) { 
    String str;
    if (!stream && !isVideo()) {
      album.setXmlParam("display", "");
      str = getPlayitemMetaDataString("album");
      String l = getPlayitemMetaDataString("track");
      if (l != "" && l != "-1") str += " (Track " + l + ")";
      album.setText(str); 
      year.setText(getPlayitemMetaDataString("year"));
    } else {
      album.setText("");
      album.setXmlParam("display", "songinfo");
    }
    album.show(); 
  }
  maxv = artist.getAutoWidth();
  if (maxv < album.getAutoWidth()) maxv = album.getAutoWidth();
  if (maxv < _title.getAutoWidth()) maxv = _title.getAutoWidth();
  if (maxv < 128) maxv = 128;
  if (maxv > getViewportWidth()/2) maxv = getViewportWidth()/2;
  if (corneroverride != "") {
    if (maxv < (nexttrack.getAutoWidth() + 50)) maxv = nexttrack.getAutoWidth() + 50;
  }
  
  return maxv;
}

// ------------------------------------------------------------------------------
Int fillCustomInfo(String customtext) {
  Group p = notifier_layout;
  Text plentry = p.findObject("plentry");
  Text nexttrack = p.findObject("nexttrack");
  Text _title = p.findObject("title");
  Text album = p.findObject("album");
  Text artist = p.findObject("artist");
  Text year = p.findObject("year");
  Text endofplayback = p.findObject("endofplayback");

  if (plentry) { plentry.hide(); }
  if (nexttrack) nexttrack.hide();
  if (_title) { _title.hide(); }
  if (artist) { artist.hide(); }
  if (album) { album.hide(); }
  if (year) { year.hide(); }

  if (endofplayback) {
    endofplayback.setText(customtext);
    int aw = endofplayback.getAutoWidth();
    endofplayback.show();
    if (aw > 128)
      return aw;
  }
  return 128;
}