/*************************************************************

  KameleonDUI - progbutton.m
  by Leechbite

  script for the programmable button.
  
  param = "buttonID,orogbuttonID,default"
  
  buttonID - button ID. im using right now XUI objects so should be the XUI id.
  progbuttonID - used to identify one programmable button from another. mainly
  				used for the privateString variables. buttons with same
  				progbuttonID will have same programmed function.
  default - default programmed button.
  			e.g. "EQ|Rock" or "PATH|http://www.leechbite.com/radio.nsv|Station Name"
  
*************************************************************/

#include <lib/std.mi>
#include <lib/pldir.mi>
#include "attribs.m"
#include "utilities.m"
#include "eq-utilities.m"

function string getCurrProg();
function bookmarkCurrent();
function addpathURL();
function setToEQPreset(int preset);
function setToolTip();

function string pathToURL(string path); // converts local path to URL, nasty workaraound for a WA bug. thanks sLob.

Global group scriptGroup;
Global button progbutton;
Global string buttonID, defaultProg;

Global container queryPathURLCont;
Global layout queryPathURLLayout;
Global button queryPathURLOKButton;
Global edit queryPathURLEditBox,queryNameEditBox;

System.onScriptLoaded() {
	initAttribs();

	scriptGroup = getScriptGroup();
	string param = getParam();
	
	progbutton = findXUIObject(scriptGroup, getToken(param,",",0),"xuibutton");
	
	buttonID = getToken(param,",",1);
	defaultProg = getToken(param,",",2);
	
	setToolTip();
	
}

System.onScriptUnloading() {
	return;
}

scriptGroup.onSetVisible(int on) {
	if (on) setTooltip();
}

string getCurrProg() {
	string currProg = getPrivateString(getSkinName(), buttonID, "Default");
	if (currProg=="Default") currProg = defaultProg;
	string ret = "";
	string progmode = strupper(getToken(currProg, "|", 0));
	
	
	if (progmode == "PATH") {
		string path = getToken(currProg, "|", 1);
		if (strlen(path) > 30) {
			path = strleft(path, 13)+" .. "+removePath(path);
		}
		ret = "PATH - " + path;
	} else if (progmode == "EQ") {
		ret = "EQ Preset - "+getToken(currProg, "|", 1);
	} else if (progmode == "INET") {
		string mode = getToken(currProg, "|", 1);
		if (mode=="MOREINFO")
			ret = "Artist/Album Search";
		else if (mode=="LYRICS")
			ret = "Lyrics Search";
		else if (mode=="CHORDS")
			ret = "Chords Search";
		else if (mode=="OFFICIALSITE")
			ret = "Artist Official Site";
	} else if (progmode == "MLPL") {
		ret = "Media Lib Playlist: "+getToken(currProg, "|", 1);
	}
	
	return ret;
}


string pathToURL(string path) {
	int c, len = strlen(path);
	string ret = "", char;

	
	c=0;
	while (c < len) {
		char = strmid(path,c,1);
		if (char==chr(92)) {
			ret = ret + chr(47);
			c ++;
		} else if (char==":") { //for ":/"
			ret = ret + ":";
			c ++;
			
			char = strmid(path,c,1);
			if (char==chr(92)) {
				ret = ret + chr(47) + chr(47);
				c ++;
			}
		} else {
			ret = ret + char;
			c++;
		}
	}
	
	return ret;
}

progbutton.onLeftClick() {
	string currProg = getPrivateString(getSkinName(), buttonID, "Default");
	if (currProg=="Default") currProg = defaultProg;
	string progmode = strupper(getToken(currProg, "|", 0));
	
	if (currProg == "Default") {
		string path = getToken(defaultProg, "|", 1);
		playFile(pathToURL(path));
	} else if (progmode == "PATH") {
		string path = getToken(currProg, "|", 1);
		playFile(pathToURL(path));
	} else if (progmode == "EQ") {
		string EQPreset = strupper(getToken(currProg, "|", 1));
		
		if (EQPreset == "FULL") {
			setEQPreset(EQ_Full);
			setTempText("EQ : Full");
		} else if (EQPreset == "ROCK") {
			setEQPreset(EQ_Rock);
			setTempText("EQ : Rock");
		} else if (EQPreset == "SOFT") {
			setEQPreset(EQ_Soft);
			setTempText("EQ : Soft");
		} else if (EQPreset == "POP") {
			setEQPreset(EQ_Pop);
			setTempText("EQ : Pop");
		} else if (EQPreset == "USER") {
			setEQPresetUSER();
			setTempText("EQ : USER");
		} else {
			setEQPreset(EQ_Flat);
			setTempText("EQ : Flat");
		}
		
		if (!getEQ()) setEQ(ON);
	} else if (progmode == "INET") {
		inetbrowser_openpage_attrib.setData(getToken(currProg, "|", 1));
	} else if (progmode == "MLPL") {
		string plName = getToken(currProg, "|", 1);
		int plID = stringtointeger(getToken(currProg, "|", 2));
		int numPL = PlDir.getNumItems();
		int found = FALSE;
		
		if (plID > (numPL-1)) plID = numPL-1;
		
		if (numPL > 0) {
			if (PlDir.getItemName(plID) == plName) {
				PlDir.playItem(plID);
				found = TRUE;
			} else {
				int c;
				
				for (c=1; c <=numPL; c++) {
	
					if ((PlDir.getItemName(c-1) == plName) && (!found)) {
						PlDir.playItem(c-1);
						found = TRUE;
					}
				}
	
			}
		}
		if (!found) settemptext("Playlist not found");
	}
}

progbutton.onRightButtonUp(int x, int y) {
	string tooltip = getXMLParam("tooltip");
	setXMLParam("tooltip","");
	
	popupMenu progMenu, eqProgSubMenu, inetSubMenu, mlplSubMenu, shoutcastSubMenu;
	int c;
	

	eqProgSubMenu = new popupMenu;
	eqProgSubMenu.addCommand("- EQ Presets -", -1, 0, 1);
	eqProgSubMenu.addSeparator();
	eqProgSubMenu.addCommand(" Flat", 100, 0, 0);
	eqProgSubMenu.addCommand(" Full", 101, 0, 0);
	eqProgSubMenu.addCommand(" Rock", 102, 0, 0);
	eqProgSubMenu.addCommand(" Soft", 103, 0, 0);
	eqProgSubMenu.addCommand(" Pop", 104, 0, 0);
	eqProgSubMenu.addCommand(" USER", 105, 0, 0);
	
	inetSubMenu = new popupMenu;
	inetSubMenu.addCommand("- Internet Search -", -1, 0, 1);
	inetSubMenu.addSeparator();
	inetSubMenu.addCommand(" Artist/Album Info", 300, 0, 0);
	inetSubMenu.addCommand(" Song Lyrics", 301, 0, 0);
	inetSubMenu.addCommand(" Song Chords", 302, 0, 0);
	inetSubMenu.addCommand(" Artist Official Site", 303, 0, 0);
	
	mlplSubMenu = new popupMenu;
	mlplSubMenu.addCommand("- Playlists -", -1, 0, 1);
	mlplSubMenu.addSeparator();
	if (PlDir) {
		int numPL = PlDir.getNumItems();
		if (numPL<=0)
			mlplSubMenu.addCommand("no playlist found", -1, 0, 1);
		else {
			for (c=1; c <=numPL; c++) {
				mlplSubMenu.addCommand(PlDir.getItemName(c-1),399+c, 0, 0);
			}
		}
	} else {
		mlplSubMenu.addCommand("feature requires WA 5.51+", -1, 0, 1);
	}
	
	shoutcastSubMenu = new popupMenu;
	shoutcastSubMenu.addCommand("- SHOUTCast Radio -", -1, 0, 1);
	shoutcastSubMenu.addSeparator();
	shoutcastSubMenu.addCommand("SHOUTCast Homepage", 599, 0, 0);
	shoutcastSubMenu.addSeparator();
	
	string shoutcastlistID = "shoutcast.list";
	string currsc = "";
	if (getString("shoutcast.list.update",0)!="") shoutcastlistID = "shoutcast.list.update";
	int c = 0;
	
	do {
		currsc = getString(shoutcastlistID,c);
		if (currsc!="") {
			shoutcastSubMenu.addCommand(" "+getToken(currsc,"|",1),500+c, 0, 0);
		}
		c++;
	} while (currsc!="");
	
	progMenu = new popupMenu;
	progMenu.addCommand("  - Program Button : "+buttonID+" -", -1, 0, 1);
	progMenu.addCommand(" Current mode : " + getCurrProg(), -2, 0, 1);
	progMenu.addSeparator();
	progMenu.addCommand(" Bookmark current song", 1, 0, 0);
	progMenu.addCommand(" Type path/url", 2, 0, 0);
	progMenu.addSubMenu(eqProgSubMenu," Equalizer Preset");
	progMenu.addSubMenu(inetSubMenu," Internet Search");
	progMenu.addSubMenu(mlplSubMenu," Playlists");
	progMenu.addSubMenu(shoutcastSubMenu," SHOUTCast Radio");
	progMenu.addCommand(" Reset to default", 99, 0, 0);
	
	int res = progMenu.popAtMouse();
	delete mlplSubMenu;
	delete inetSubMenu;
	delete eqProgSubMenu;
	delete shoutcastSubMenu;
	delete progMenu;
	
	if (res == 1) {
		bookmarkCurrent();
	} else if (res == 2) {
		addpathURL();
	} else if (res >= 100 && res < 200) {
		setToEQPreset(res);
	} else if (res == 300) {
		setPrivateString(getSkinName(), buttonID, "INET|MOREINFO");
	} else if (res == 301) {
		setPrivateString(getSkinName(), buttonID, "INET|LYRICS");
	} else if (res == 302) {
		setPrivateString(getSkinName(), buttonID, "INET|CHORDS");
	} else if (res == 303) {
		setPrivateString(getSkinName(), buttonID, "INET|OFFICIALSITE");
	} else if (res == 99) {
		setPrivateString(getSkinName(), buttonID, "Default");
	} else if (res >= 400 && res < 500) {
		setPrivateString(getSkinName(), buttonID, "MLPL|"+PlDir.getItemName(res-400)+"|"+integerToString(res-400));
	} else if (res >= 500 && res < 599) {
		setPrivateString(getSkinName(), buttonID, "PATH|"+getString(shoutcastlistID,res-500));
	} else if (res == 599) {
		setPrivateString(getSkinName(), buttonID, "INET|http://www.shoutcast.com/");
	}

	setToolTip();
		
	complete;
	
}

setToolTip() {
	string currProg = getPrivateString(getSkinName(), buttonID, defaultProg);
	if (currProg=="Default") currProg = defaultProg;
	string progmode = strupper(getToken(currProg, "|", 0));
	string tip = "";
	
	if (progmode == "PATH") {
		string path = getToken(currProg, "|", 1);
		string protocol = strlower(getToken(path, "://", 0));
		tip = getToken(currProg, "|", 2);
		if (tip=="") {
			if (protocol == "http" || protocol == "mms")
				tip = "Internet Radio";
			else
				tip = removePath(path);
		}
	} else if (progmode == "EQ") {
		tip = "EQ Preset: "+getToken(currProg, "|", 1);
	} else if (progmode == "INET") {
		string mode = getToken(currProg, "|", 1);
		if (mode=="MOREINFO")
			tip = "Artist/Album Search";
		else if (mode=="LYRICS")
			tip = "Lyrics Search";
		else if (mode=="CHORDS")
			tip = "Chords Search";
		else if (mode=="OFFICIALSITE")
			tip = "Artist Official Site";
		else
			tip = mode;
	} else if (progmode == "MLPL") {
		tip = "Playlist: "+getToken(currProg, "|", 1);
	}
	
	progButton.setXMLParam("tooltip", tip+" | Right-click to program button");
}

bookmarkCurrent() {
	string currPath = System.getPlayItemString();
	
	setPrivateString(getSkinName(), buttonID, "PATH|"+currPath);
}

addpathURL() {
	if (isObjectValid(queryPathURLCont)) { // if already open, dont reopen
		layout queryPathURLLayout = queryPathURLCont.enumLayout(0);
		queryPathURLLayout.setFocus();
		queryPathURLLayout.center();
		return;
	}
	
	queryPathURLCont = newDynamicContainer("query.pathurl");
	if (queryPathURLCont) {
		queryPathURLCont.show();
		queryPathURLLayout = queryPathURLCont.getLayout("normal");
		queryPathURLLayout.center();
		queryPathURLOKButton = queryPathURLLayout.findObject("ok");
		queryPathURLEditBox = queryPathURLLayout.findObject("pathurl.editbox");
		queryNameEditBox = queryPathURLLayout.findObject("medianame.editbox");
	}
}

queryPathURLOKButton.onLeftClick() {
	if (queryPathURLEditBox.getText()=="") return;
	setPrivateString(getSkinName(), buttonID, "PATH|"+queryPathURLEditBox.getText()+"|"+queryNameEditBox.getText());
	queryPathURLCont.close();
	queryPathURLCont = NULL;
	
	setToolTip();
}

queryPathURLEditBox.onEnter() {
	queryNameEditBox.setFocus();
}

queryNameEditBox.onEnter() {
	queryPathURLOKButton.onLeftClick();
}

setToEQPreset(int preset) {
	string EQPresetName = "";
	if (preset == 101)
		EQPresetName = "Full";
	else if (preset == 102)
		EQPresetName = "Rock";
	else if (preset == 103)
		EQPresetName = "Soft";
	else if (preset == 104)
		EQPresetName = "Pop";
	else if (preset == 105)
		EQPresetName = "USER";
	else
		EQPresetName = "Flat";
		
	setPrivateString(getSkinName(), buttonID, "EQ|"+EQPresetName);
}