/*************************************************************

  KameleonDUI - wheel.m
  by Leechbite

  -Controls the volume/multipurpose wheel

*************************************************************/

#include <lib/std.mi>
#include "attribs.m"
#include "utilities.m"

#define WHEEL_VOL 0
#define WHEEL_SEEKER 1
#define WHEEL_BASS 2
#define WHEEL_TREBLE 3
#define WHEEL_BALANCE 4

function setTreble(int val);
function setBass(int val);
function int getWheelMode();
function setupWheel();

Global group scriptGroup;
Global map wheelMap;
Global animatedLayer wheel;
Global layer wheelMouseTrap;

Global int wheeladjust;
Global int lastvol ,lastpos,  lastbal, lastbass, lasttreble, lastwheelval;
Global int wheelMode = WHEEL_VOL;

Global slider balslider;

System.onScriptLoaded() {
  initAttribs();

  scriptGroup = getScriptGroup();
  wheel = scriptGroup.getObject("wheel");
  wheelMouseTrap = scriptGroup.getObject("wheel.trap");
  balslider = getContainer("EQ").getLayout("normal").findObject("bal.slider");

  wheelMap = new map;
  wheelMap.loadmap("main.wheel.map");

  wheeladjust = 0;
  
  setupWheel();
}

System.onScriptUnloading() {
  delete wheelMap;
}

setupWheel() {
	wheelMode = getWheelMode();
	
	if (wheelmode == WHEEL_VOL) {
		wheelMouseTrap.setXMLParam("tooltip", "Volume | Right-click to change wheel function");
	}
	if (wheelmode == WHEEL_SEEKER) {
		wheelMouseTrap.setXMLParam("tooltip", "Seek | Right-click to change wheel function");
	}
	if (wheelmode == WHEEL_BASS) {
		wheelMouseTrap.setXMLParam("tooltip", "Bass | Right-click to change wheel function");
	}
	if (wheelmode == WHEEL_TREBLE) {
		wheelMouseTrap.setXMLParam("tooltip", "Treble | Right-click to change wheel function");
	}
	if (wheelmode == WHEEL_BALANCE) {
		wheelMouseTrap.setXMLParam("tooltip", "Balance | Right-click to change wheel function");
	}
}

wheelMouseTrap.onLeftButtonDown(int x, int y) {
  wheeladjust = 1;

  lastwheelval = wheelmap.getValue(x-getLeft(), y-getTop());

  lastvol = getVolume();
  lastpos = getPosition();
  lastbass = getPrivateInt(getSkinName(), "Last Bass", 127);
  lasttreble = getPrivateInt(getSkinName(), "Last Treble", 127);
  
  if (isKeyDown(VK_SHIFT) && isKeyDown(VK_CONTROL)) wheelMode = WHEEL_BASS;
  else if (isKeyDown(VK_CONTROL) && isKeyDown(VK_ALT)) wheelMode = WHEEL_TREBLE;
  else if (isKeyDown(VK_SHIFT)) wheelMode = WHEEL_VOL;
  else if (isKeyDown(VK_CONTROL)) wheelMode = WHEEL_SEEKER;
  else if (isKeyDown(VK_ALT)) wheelMode = WHEEL_BALANCE;
}

wheelMouseTrap.onLeftButtonUp(int x, int y) {
  if (wheeladjust) {
    if (wheelmode == WHEEL_SEEKER) {
      seekto(lastpos);
    }
    if (wheelmode == WHEEL_BASS) {
      setBass(getPrivateInt(getSkinName(), "Last Bass", 127));
    }
    if (wheelmode == WHEEL_TREBLE) {
      setTreble(getPrivateInt(getSkinName(), "Last Treble", 127));
    }
  }
  wheeladjust = 0;
  
  wheelMode = getWheelMode();
}

wheelMouseTrap.onMouseMove(int x, int y) {
  if (wheeladjust) {
    int rx = x-getLeft(); int ry = y-getTop();
    if ((rx<0) || (rx<0) || (rx>=getWidth()) || (ry>=getHeight()))
	return;

    int newwheelval = wheelmap.getValue(rx, ry);
    int wheelangle =newwheelval-lastwheelval;
    int adjust = wheelangle;

    if (wheelangle<0) wheelangle=255+wheelangle;

    wheelangle = wheelangle * 360/255;
    int frame = (wheelangle % 45)*wheel.getLength()/44;

    wheel.gotoFrame(frame);

    if (wheelmode == WHEEL_VOL) {
      boolean resetvol = 0;
      if (adjust > 100) {
        adjust = -(255-adjust+lastwheelval);
        resetvol = 1;
      }
      if (adjust < -100) {
        adjust = (255-lastwheelval)+(255+adjust);
        resetvol = 1;
      }

      int newvol = lastvol + adjust;
      if (newvol < 0) newvol = 0;
      if (newvol > 255) newvol = 255;

      setVolume(newvol);
      if ((adjust>50) || (adjust<-50) || (resetvol)) {
		lastvol = newvol;
		lastwheelval = newwheelval;
      }
    } else if (wheelmode == WHEEL_SEEKER) {
      boolean resetseeker = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resetseeker = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resetseeker = 1;
      }

      lastpos = lastpos + adjust*50;
      if (lastpos < 0) lastpos = 0;
      if (lastpos > getPlayItemLength()) lastpos =  getPlayItemLength();

      float len = System.getPlayItemLength();
	  setTempText("Seek: " + integerToTime(lastpos) + "/" + integerToTime(len) + " ("+integerToString(lastpos*100/len)+"%)");

      if ((adjust>50) || (adjust<-50) || (resetseeker)) {
		lastwheelval = newwheelval;
      }
    } else if (wheelmode == WHEEL_BASS) {
      boolean resetbass = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resetbass = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resetbass = 1;
      }

      int newbass = lastbass + adjust;
      if (newbass < 0) newbass = 0;
      if (newbass > 255) newbass =  255;

      int pr;
      pr = newbass-127;
      pr = pr*100/127;

      setTempText("Bass: "+integertostring(pr)+"%");
      setPrivateInt(getSkinName(), "Last Bass", newbass);

      if ((adjust>50) || (adjust<-50) || (resetbass)) {
        lastbass = newbass;
	lastwheelval = newwheelval;
      }
    } else if (wheelmode == WHEEL_TREBLE) {
      boolean resettreble = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resettreble = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resettreble = 1;
      }

      int newtreble = lasttreble + adjust;
      if (newtreble < 0) newtreble = 0;
      if (newtreble > 255) newtreble =  255;

      int pr;
      pr = newtreble-127;
      pr = pr*100/127;

      setTempText("Treble: "+integertostring(pr)+"%");
      setPrivateInt(getSkinName(), "Last Treble", newtreble);

      if ((adjust>50) || (adjust<-50) || (resettreble)) {
        lasttreble = newtreble;
		lastwheelval = newwheelval;
      }
    } else if (wheelmode == WHEEL_BALANCE) {
      boolean resetbalance = 0;
      if (adjust > 200) {
        adjust = -(255-adjust+lastwheelval);
        resetbalance = 1;
      }
      if (adjust < -200) {
        adjust = (255-lastwheelval)+(255+adjust);
        resetbalance = 1;
      }

      int newbal = lastbal + adjust;
      if (newbal < 0) newbal = 0;
      if (newbal > 255) newbal =  255;
      if ((newbal > 120) && (newbal < 133)) newbal = 127; // snapping to center code

      balslider.setPosition(newbal);
      int pr;
      pr = newbal-127;
      pr = pr*100/127;

      if (pr>0) {
		setTempText("Bal Right: "+integertostring(pr)+"%");
      } else if (pr<0) {
		setTempText("Bal Left: "+integertostring(-pr)+"%");
      } else {
		setTempText("Bal Center");
      }

      if ((adjust>50) || (adjust<-50) || (resetbalance)) {
		lastbal = newbal;
		lastwheelval = newwheelval;
      }
    }
  }
}

wheelMouseTrap.onRightButtonDown(int x, int y)  {
  popupMenu wheelMenu;

  wheelMenu = new popupMenu;
  wheelMenu.addCommand("- Wheel Control -", -1, 0, 1);
  wheelMenu.addSeparator();
  wheelMenu.addCommand(" Volume \tSHFT-click", WHEEL_VOL, wheelmode==WHEEL_VOL, 0);
  wheelMenu.addCommand(" Seeker \tCTRL-click", WHEEL_SEEKER, wheelmode==WHEEL_SEEKER, 0);
  wheelMenu.addCommand(" Bass \tSHFT-CTRL-click", WHEEL_BASS, wheelmode==WHEEL_BASS, 0);
  wheelMenu.addCommand(" Treble \tCTRL-ALT-click", WHEEL_TREBLE, wheelmode==WHEEL_TREBLE, 0);
  wheelMenu.addCommand(" Balance \tALT-click", WHEEL_BALANCE, wheelmode==WHEEL_BALANCE, 0);

  setXMLParam("tooltip", "");

  int res = wheelMenu.popAtMouse();
  delete wheelMenu;

  if (res<0) return;
  
  if (res == WHEEL_VOL) wheel_vol_attrib.setData("1");
  if (res == WHEEL_SEEKER) wheel_seek_attrib.setData("1");
  if (res == WHEEL_BASS) wheel_bass_attrib.setData("1");
  if (res == WHEEL_TREBLE) wheel_treble_attrib.setData("1");
  if (res == WHEEL_BALANCE) wheel_bal_attrib.setData("1");

  complete; 
}

// *********** treble/bass control scripts ***************
SetTreble(int val) {
  val -= 127;

  setEq(1);

  int move;
  for(int i=5; i<10; i++) {
    move = val*1.27/(10-i);
    setEqBand(i, move);
  }
}

SetBass(int val) {
  val -=127;

  setEq(1);

  int move;
  for(int i=0; i<5; i++) {
    move = val*1.27/(i+1);
    setEqBand(i, move);
  }
}

int getWheelMode() {
  if (wheel_vol_attrib.getData()=="1") return WHEEL_VOL;
  if (wheel_bass_attrib.getData()=="1") return WHEEL_BASS;
  if (wheel_treble_attrib.getData()=="1") return WHEEL_TREBLE;
  if (wheel_seek_attrib.getData()=="1") return WHEEL_SEEKER;
  if (wheel_bal_attrib.getData()=="1") return WHEEL_BALANCE;
}

wheel_vol_attrib.onDataChanged() {
	if (getData()=="1") setupWheel();
}

wheel_seek_attrib.onDataChanged() {
	if (getData()=="1") setupWheel();
}

wheel_bass_attrib.onDataChanged() {
	if (getData()=="1") setupWheel();
}

wheel_treble_attrib.onDataChanged() {
	if (getData()=="1") setupWheel();
}

wheel_bal_attrib.onDataChanged() {
	if (getData()=="1") setupWheel();
}